
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit
import AVFoundation

class cAVAudioPlayer: NSObject
{
    var mPlayerWave: AVAudioPlayer?
    
    var mTargetPlayDone: AnyObject?
    var mActionPlayDone: Selector?
    
    var mTargetPlayTick: AnyObject?
    var mActionPlayTick: Selector?
    
    var mTimer: Timer?
    
   
    override init()
    {
        mTimer = nil
        mPlayerWave = nil
        
        mActionPlayDone = nil
        mTargetPlayDone = nil
        
        mTargetPlayTick = nil
        mActionPlayTick = nil
        
        super.init()
    }
    
    deinit
    {
        self.PlayUnload()
    }
    
    func Play(_ nLoopCount: Int) ->Void
    {
        mPlayerWave?.delegate = self as? AVAudioPlayerDelegate
        mPlayerWave?.numberOfLoops = nLoopCount
        mPlayerWave?.volume = 1.0
        
        mPlayerWave?.play()
    }
    
    func Stop() ->Void
    {
        mPlayerWave?.stop()
    }
    
    func Pause() ->Void
    {
        mPlayerWave?.pause()
    }

    func PlayUnload() ->Void
    {
        if (mPlayerWave != nil)
        {
            self.Stop()
            mPlayerWave = nil
        }
        
        if (mTimer != nil)
        {
            mTimer?.invalidate()
            mTimer = nil
        }
    }
    
    @objc func OnTimerTick(vTimer: Timer) ->Void
    {
        if mTargetPlayTick != nil
        {
            mTargetPlayTick?.perform(mActionPlayTick!, with: self, afterDelay: 0.0)
        }
    }
    
    func IsPlaying() ->Bool
    {
        if mPlayerWave == nil
        {
            return false
        }
        
        return mPlayerWave!.isPlaying
    }
    
    func PlayStart(_ nLoopCount: Int) ->Void
    {
        if mPlayerWave == nil
        {
            return
        }
        
        self.Play(nLoopCount)
        
        if (mTimer != nil)
        {
            mTimer?.invalidate()
            mTimer = nil
        }
        
        if mTargetPlayTick != nil
        {
            mTimer = Timer.scheduledTimer(timeInterval: 0.1, target: self, selector: #selector(self.OnTimerTick), userInfo: self, repeats: true)
        }
    }
    
    func PlayStop() ->Void
    {
        if mPlayerWave == nil
        {
            return
        }
        
        if (mTimer != nil)
        {
            mTimer?.invalidate()
            mTimer = nil
        }
        
        self.Stop()
    }
    
    func PlayPause() ->Void
    {
        if mPlayerWave == nil
        {
            return
        }
        
        if (mTimer != nil)
        {
            mTimer?.invalidate()
            mTimer = nil
        }
        
        self.Pause()
    }
    
    func PlayLoadWithData(_ vFileData: NSData)
    {
        self.PlayUnload()
        
        mPlayerWave = try? AVAudioPlayer(data: vFileData as Data)
    }
    
    func IsFileExist(_ vFileName: String) -> Bool
    {
        return FileManager.default.fileExists(atPath: vFileName)
    }
    
    func PlayLoadWithFile(_ vFileName: String) ->Void
    {
        self.PlayUnload()
        
        if !IsFileExist(vFileName)
        {
            return
        }
        
        mPlayerWave = try? AVAudioPlayer(contentsOf: URL(fileURLWithPath: vFileName))
    }
    
    func audioPlayerDidFinishPlaying(_ player: AVAudioPlayer, successfully flag: Bool)
    {
        self.PlayPause()
        
        if mTargetPlayDone != nil
        {
            mTargetPlayDone?.perform(mActionPlayDone!, with: self, afterDelay: 0.0)
        }
    }
    
    func GetTotalPlayTime() ->TimeInterval
    {
        if mPlayerWave == nil
        {
            return 0
        }
        
        return mPlayerWave!.duration
    }
    
    func GetCurrentTime() ->TimeInterval
    {
        if mPlayerWave == nil
        {
            return 0
        }
        
        return mPlayerWave!.currentTime
    }
    
    func PlayAtTime(_ nTime: TimeInterval) ->Void
    {
        if mPlayerWave == nil
        {
            return
        }
        
        mPlayerWave?.currentTime = nTime
    }
    
    func SetPlayDoneDelegate(_ TargetOBJ: AnyObject, _ ActionDONE: Selector) ->Void
    {
        mTargetPlayDone = TargetOBJ
        mActionPlayDone = ActionDONE
    }
    
    func SetPlayTickDelegate(_ TargetOBJ: AnyObject, _ ActionTICK: Selector) ->Void
    {
        mTargetPlayTick = TargetOBJ
        mActionPlayTick = ActionTICK
    }
}
