
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit
import Contacts
import ContactsUI

@objc protocol cAddressBookViewDelegate: NSObjectProtocol
{
    @objc optional func OnSelectedPhoneNo(_ sPhoneNo: String) ->Void
    @objc optional func OnSelectedContact(_ sConatctId: String, _ sContactTitle: String) ->Void
}

class cAddressBookView: NSObject, CNContactPickerDelegate
{
    var m_bGranted: Bool
    var m_bAccessOnlyContacts: Bool
    
    var mAddressBook: CNContactPickerViewController?
    var mAddressBookDelegate: cAddressBookViewDelegate!
    
    init(iDelegate: Any)
    {
        mAddressBookDelegate = (iDelegate as! cAddressBookViewDelegate)

        mAddressBook = nil
        m_bAccessOnlyContacts = false
        m_bGranted = false
        
        super.init()
    }
    
    deinit
    {
        self.HideAddressBookUI()
    }
    
    func didReceiveMemoryWarning()
    {
    }
    
    func contactPicker(_ picker: CNContactPickerViewController, didSelect contact: CNContact)
    {
        self.OnSelectedContact(contact.identifier)
    }
    
    func contactPicker(_ picker: CNContactPickerViewController, didSelect contactProperty: CNContactProperty)
    {
       // let contact: CNContact = contactProperty.contact
        
        let contactPhone: CNPhoneNumber = contactProperty.value as! CNPhoneNumber
        let phoneNumber: String = contactPhone.stringValue
        
        self.OnSelectedPhoneNo(phoneNumber)
    }
    
    func contactPickerDidCancel(_ picker: CNContactPickerViewController)
    {
        
    }
    
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
    
    private func ChangeNavigationBar() ->Void
    {
         UINavigationBar.appearance().tintColor = UIColor.darkGray
    }
    
    func ShowContacts(_ Owner: UIViewController, _ bAccessOnlyContacts: Bool) ->Void
    {
        self.ChangeNavigationBar()
        self.ShowAddressBookUI(Owner, bAccessOnlyContacts)
    }
    
    func OnAddressBookAccess(pParent: UIViewController) ->Void
    {
        if m_bGranted
        {
            self.CloseAddressBook()
            return
        }
        
        pParent.present(mAddressBook!, animated: true, completion: nil)
    }
    
    func ShowAddressBookUI(_ Owner: UIViewController, _ bAccessOnlyContacts: Bool) ->Bool
    {
        mAddressBook = CNContactPickerViewController()
        mAddressBook?.delegate = self
        
        mAddressBook?.displayedPropertyKeys = [CNContactPhoneNumbersKey]
        
        m_bAccessOnlyContacts = bAccessOnlyContacts
        
        Owner.present(mAddressBook!, animated: true, completion: nil)
        
        if bAccessOnlyContacts == false
        {
            mAddressBook?.predicateForSelectionOfContact = NSPredicate(value: false)
        }
        
        return true
    }
    
    func HideAddressBookUI() ->Void
    {
        mAddressBook = nil
    }
    
    @objc func OnDimissCompleted() ->Void
    {
        self.HideAddressBookUI()
    }
    
    func CloseAddressBook() ->Void
    {
        mAddressBook?.dismiss(animated: true, completion:{() -> Void in
                
        self.performSelector(onMainThread: #selector(self.OnDimissCompleted), with: nil, waitUntilDone: false)
        })
    }
    
    ////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////
    
    func OnSelectedPhoneNo(_ sPhoneNo: String) ->Void
    {
        if mAddressBookDelegate == nil
        {
            return
        }
        
        if !mAddressBookDelegate.responds(to: #selector(cAddressBookViewDelegate.OnSelectedPhoneNo))
        {
            return
        }
        
        mAddressBookDelegate.OnSelectedPhoneNo!(sPhoneNo)
        self.CloseAddressBook()
        
        
    }
    
    func OnSelectedContact(_ sContactId: String)
    {
        if mAddressBookDelegate == nil
        {
            return
        }
        
        if !mAddressBookDelegate.responds(to: #selector(cAddressBookViewDelegate.OnSelectedContact))
        {
            return
        }
        
        let vBook = cAddressBook()
        let sTitle: String = vBook.ReadContactName(sContactId)
        
        mAddressBookDelegate.OnSelectedContact!(sContactId, sTitle)
        self.CloseAddressBook()
    }
    
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

}
