
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

@objc protocol iVaxBackForeNotify: NSObjectProtocol
{
    @objc optional func OnVaxApplicationEnterBackground() ->Void
    @objc optional func OnVaxApplicationEnterForeground() ->Void
}

@objc class cVaxBackForeNotify : NSObject
{
    var mBackForeDelegate: iVaxBackForeNotify!
    var m_bForeground: Bool = true;
    
    init(iDelegate: Any)
    {
        m_bForeground = true;
        mBackForeDelegate = (iDelegate as! iVaxBackForeNotify)
        
        super.init()
    }

    deinit
    {
        
    }
    
    //////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////
    
    func BackForegroundNotify(_ bEnable: Bool) ->Void
    {
        m_bForeground = true;
        
        BackgroundNotify(bEnable);
        ForegroundNotify(bEnable);
    }
    
    //////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////
    
    private func ForegroundNotify(_ bEnable: Bool) ->Void
    {
        NotificationCenter.default.removeObserver(self, name: UIApplication.didBecomeActiveNotification, object: nil)
        
        if bEnable
        {
            NotificationCenter.default.addObserver(self, selector: #selector(OnApplicationEnterForeground), name: UIApplication.didBecomeActiveNotification, object: nil)
        }
    }
    
    private func BackgroundNotify(_ bEnable: Bool) ->Void
    {
        NotificationCenter.default.removeObserver(self, name: UIApplication.willResignActiveNotification, object: nil)
        
        if bEnable
        {
            NotificationCenter.default.addObserver(self, selector: #selector(OnApplicationEnterBackground), name: UIApplication.willResignActiveNotification, object: nil)
        }
    }
            
    //////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////
    
    @objc private func OnApplicationEnterForeground()
    {
        if m_bForeground
        {
            return;
        }
        
        m_bForeground = true;
        
        if !mBackForeDelegate.responds(to: #selector(iVaxBackForeNotify.OnVaxApplicationEnterForeground))
        {
            return;
        }
        
        mBackForeDelegate.OnVaxApplicationEnterForeground!()
    }
    
    @objc private func OnApplicationEnterBackground()
    {
        if !m_bForeground
        {
            return;
        }
        
        m_bForeground = false;
        
        if !mBackForeDelegate.responds(to: #selector(iVaxBackForeNotify.OnVaxApplicationEnterBackground))
        {
            return;
        }
        
        mBackForeDelegate.OnVaxApplicationEnterBackground!()
    }
}
