//
//  AppDelegate.swift
//  VaxPhone
//
//  Created by VaxSoft on 04/07/2017.
//  Copyright © 2025 VaxSoft. All rights reserved.
//

import UIKit
import UserNotifications

var m_nMainAppMissedCount: Int = 0
var m_bMainAppBackground: Bool = true


@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate
{
    var mVaxVoip: cVaxPhoneSIP!
    var window: UIWindow?
    
    static func LoadDataAll() ->Void
    {
        cDataSQL.CheckAndCreateDatabase()
        
        cVideoView.OnStartedAPP()
        
        cChatContactTableView.OnStartedAPP()
    }
    
    static func ResetSettings() ->Void
    {
        cVideoView.ResetData()
        
        cSettingMicSpkTableView.ResetData()
        cSettingNetworkTableView.ResetData()
                
        cSettingVoiceChangerTableView.ResetData()
        cSettingVoiceCodecTableView.ResetData()
        
        cSettingRingToneTableView.ResetData()
        cSettingVideoCodecTableView.ResetData()
        cSettingDigitDTMFTableView.ResetData()
        
        cSettingVideoQualityTableView.ResetData()
        cSettingVideoBitrateTableView.ResetData()
    }
    
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
    
    static func PostChatMsgRecvText() ->Void
    {
        if !m_bMainAppBackground
        {
            return
        }
        
        AppDelegate.SetAppMissedCount(nValue: -1)
    }
    
    static func PostInboundCallHistory() ->Void
    {
        if !m_bMainAppBackground
        {
            return
        }
        
        AppDelegate.SetAppMissedCount(nValue: -1)
    }
    
    static func SetAppMissedCount(nValue: Int) ->Void
    {
        if nValue == 0
        {
            m_nMainAppMissedCount = 0
        }
        else
        {
            m_nMainAppMissedCount += 1
        }
        
        UIApplication.shared.applicationIconBadgeNumber = m_nMainAppMissedCount
    }
    
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
    
    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool
    {
        
        //UIApplication.shared.isStatusBarHidden = false
        
        ///////////////////////////////////////////////
        
        cDataSQL.CheckAndCreateDatabase()
        
        let vRecentDB: cStoreRecents = cStoreRecents()
        vRecentDB.CreateTable()
        
        let vChatContactDB: cStoreChatContacts = cStoreChatContacts()
        vChatContactDB.CreateTable()
        
        let vChatMsgDB: cStoreChatMsgs = cStoreChatMsgs()
        vChatMsgDB.CreateTable()
        
        ///////////////////////////////////////////////
        
        mVaxVoip = cVaxPhoneSIP()
        
        cAddressBook.AskAcessToContacts()
        AppDelegate.LoadDataAll()
        
        self.AskPermissions()
        
        AppDelegate.SetAppMissedCount(nValue: 0)
        m_bMainAppBackground = false
        
        return true
    }
    
    func AskPermissions()
    {
        if #available(iOS 10, *)
        {
            UNUserNotificationCenter.current().getNotificationSettings
            {
                (settings) in
                
                switch settings.authorizationStatus
                {
                    
                case .notDetermined:
                    
                    UNUserNotificationCenter.current().requestAuthorization(options: [.alert, .sound, .badge], completionHandler: { (granted, error) in
                        
                        assert(error == nil)
                        
                        if granted
                        {
                            DispatchQueue.main.async {
                              UIApplication.shared.registerForRemoteNotifications()
                            }
                        }
                        else
                        {
                            // Notify user to enable push notification permission in settings
                        }
                    })
                    
                    break
                    
                case .authorized:
                    DispatchQueue.main.async {
                      UIApplication.shared.registerForRemoteNotifications()
                    }
                    break
                    
                case .denied:
                    // Notify user to enable push notification permission in settings
                    break
                    
                default:
                    return;
                    
                }
            }
        }
        
        //let types: UIUserNotificationType = [.badge, .sound, .alert]
        
        //let mySettings: UIUserNotificationSettings = UIUserNotificationSettings(types: types, categories: nil)
        //UIApplication.shared.registerUserNotificationSettings(mySettings)
    }

    func applicationWillResignActive(_ application: UIApplication)
    {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(_ application: UIApplication)
    {
         m_bMainAppBackground = true
        
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    }

    func applicationWillEnterForeground(_ application: UIApplication)
    {
        // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
        
        AppDelegate.SetAppMissedCount(nValue: 0)
        m_bMainAppBackground = false
    }

    func applicationDidBecomeActive(_ application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    }

    func applicationWillTerminate(_ application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    }
    
    
    

}
