
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import AVFoundation
import CallKit

@objc protocol iVaxCallKit: NSObjectProtocol
{
    @objc optional func OnVaxAnswerCallKit() ->Void
    @objc optional func OnVaxEndCallKit() ->Void
    @objc optional func OnVaxHoldCallKit() ->Void
    @objc optional func OnVaxDigitCallKit() ->Void
}

class cVaxCallKit: NSObject, CXProviderDelegate
{
    var mbCallKitUI: Bool;
    var mhUUID: UUID!;
        
    var mConfigCX: CXProviderConfiguration!;
    var mProviderCX: CXProvider!;
    var mUpdateCX: CXCallUpdate!;
    
    var mCallCtrl: CXCallController!;
    var mCallKitDelegate: iVaxCallKit!
    
    init(iDelegate: Any)
    {
        mCallKitDelegate = (iDelegate as! iVaxCallKit)
        
        mbCallKitUI = false;
        
        mUpdateCX = nil;
        mCallCtrl = nil;
        mProviderCX = nil;
        
        super.init()
    }
    
    deinit
    {

    }
    
    private func Destroy() ->Void
    {
        if mbCallKitUI == false
        {
            return;
        }
        
        let vCallEndDate : Date! = Date()
        
        mProviderCX.reportCall(with: mhUUID, endedAt: vCallEndDate, reason: CXCallEndedReason.remoteEnded)
        
        mbCallKitUI = false;
        
        mUpdateCX = nil;
        mCallCtrl = nil;
        mProviderCX = nil;
    }
    
    private func Create() ->Bool
    {
        self.Destroy()
        
        if (NSFoundationVersionNumber <= 1300.000000)
        {
            NSLog("Call-kit: Not supported.");
            return false;
        }
        
        mhUUID = UUID();
        mCallCtrl = CXCallController()
            
        mConfigCX = CXProviderConfiguration(localizedName: "vax")
        mConfigCX.maximumCallGroups = 1;
        mConfigCX.maximumCallsPerCallGroup = 1;
        mConfigCX.supportsVideo = false;
        
        mProviderCX = CXProvider(configuration: mConfigCX)
        mProviderCX.setDelegate(self, queue: nil)
        
        mbCallKitUI = true;
        return true;
    }
    
    func ShowUI(_ sCallerName: String!,  _ sCallerId: String!) ->Bool
    {
        if mbCallKitUI == true
        {
            return true;
        }
        
        if !self.Create()
        {
            return false;
        }
        
        var sCallerInfo: String = sCallerId;
        
        if sCallerName != sCallerId
        {
            sCallerInfo = String(format: "(%@) %@", sCallerName, sCallerId)
        }
        
        mUpdateCX = CXCallUpdate();
            
        mUpdateCX.remoteHandle = CXHandle(type: .phoneNumber, value: "VaxPhoneSDK")
            
        mUpdateCX.localizedCallerName = sCallerInfo;
        mUpdateCX.supportsHolding = true;
        mUpdateCX.supportsGrouping = false;
        mUpdateCX.supportsUngrouping = false;
        mUpdateCX.supportsDTMF = true;
        mUpdateCX.hasVideo = false;
            
        //mProviderCX.setConfiguration(mConfigCX);
        
        mProviderCX.reportNewIncomingCall(with: mhUUID, update: mUpdateCX, completion:
        {
            error in
                
            if error != nil
            {
                //self.Destroy()
            }
        })
        
        return true;
    }
    
    func HideUI()
    {
        self.Destroy()
    }
    
    ////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////
    
    func provider(_ provider: CXProvider, perform action: CXAnswerCallAction)
    {
        NSLog("Call-Kit: performAnswerCallAction");
        
        let session = AVAudioSession.sharedInstance()
        try? session.setCategory(AVAudioSession.Category.playAndRecord)
    
        action.fulfill()
    }
    
    func provider(_ provider: CXProvider, didActivate audioSession: AVAudioSession)
    {
        NSLog("Call-Kit: didActivateSession");
        
        if !mCallKitDelegate.responds(to: #selector(mCallKitDelegate.OnVaxAnswerCallKit))
        {
            return;
        }
        
        mCallKitDelegate.OnVaxAnswerCallKit!()
    }
    
    func provider(_ provider: CXProvider, perform action: CXEndCallAction)
    {
        NSLog("Call-Kit: performEndCallAction");
        
        if !mCallKitDelegate.responds(to: #selector(mCallKitDelegate.OnVaxEndCallKit))
        {
            return;
        }
        
        mCallKitDelegate.OnVaxEndCallKit!()
        
        action.fulfill()
    }
    
    /////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func provider(_ provider: CXProvider, perform action: CXSetHeldCallAction)
    {
        NSLog("Call-Kit: performHoldCallAction");
        
        if !mCallKitDelegate.responds(to: #selector(mCallKitDelegate.OnVaxHoldCallKit))
        {
            return;
        }
        
        mCallKitDelegate.OnVaxHoldCallKit!()
    
        action.fulfill()
    }
    
    func provider(_ provider: CXProvider, perform action: CXPlayDTMFCallAction)
    {
        NSLog("Call-Kit: performPlayDTMFCallAction");
        
        if !mCallKitDelegate.responds(to: #selector(mCallKitDelegate.OnVaxDigitCallKit))
        {
            return;
        }
        
        mCallKitDelegate.OnVaxDigitCallKit!()
    
        action.fulfill()
    }
    
    /////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func provider(_ provider: CXProvider, didDeactivate audioSession: AVAudioSession)
    {
        ///NSLog("didDeactivateSession");
    }

    func providerDidReset(_ provider: CXProvider)
    {
        
    }
}
