
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

@objc class cVaxPhoneRing: NSObject, iVaxUserAgentSIP
{
    var mVaxAgentSIP: cVaxUserAgentSIP!
    static var mSelf: cVaxPhoneRing?
         
    var mRingTonePLAY: cAVAudioPlayer?
    
    var mRingToneDATA: NSMutableData?
    var m_nRingToneId: Int
    
    override init()
    {
        m_nRingToneId = cVaxUserAgentInfo.VAX_RING_TONE_DIGITAL_RAIN
        
        mRingToneDATA = nil
        mRingTonePLAY = nil
        
        super.init()
        
        cVaxPhoneRing.mSelf = self
       
        mVaxAgentSIP = cVaxUserAgentSIP(iDelegate: self)
    }
    
    deinit
    {
        mRingToneDATA = nil
        mRingTonePLAY = nil
        
        mVaxAgentSIP = nil
    }
    
    static func GetSelf() ->cVaxPhoneRing
    {
        return mSelf!
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func DialRing(_ bEnable: Bool) ->Bool
    {
        if bEnable
        {
            let sFileName: String? = Bundle.main.path(forResource: "aDialRing", ofType: "wav")
            return mVaxAgentSIP.DialRingEnable(sFileName!)
        }
        
        return mVaxAgentSIP.DialRingDisable()
    }
    
    func BusyRing(_ bEnable: Bool) ->Bool
    {
        if bEnable
        {
            let sFileName: String? = Bundle.main.path(forResource: "aBusyRing", ofType: "wav")
            return mVaxAgentSIP.BusyRingEnable(sFileName!)
        }
        
        return mVaxAgentSIP.BusyRingDisable()
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////
    
    func SetRingTone(_ nRingToneId: Int) ->Void
    {
        // For devices, sample code utilizes Call-Kit framework.
        
        #if targetEnvironment(simulator)
            self.LoadRingTone(nRingToneId)
        #endif
    }
    
    func MuteRingTone() ->Void
    {
        // For devices, sample code utilizes Call-Kit framework.
        
        #if targetEnvironment(simulator)
            self.StopRingTone()
        #endif
    }
    
    func LoadRingTone(_ nRingToneId: Int) ->Void
    {
        let aFileName = ["aRingToneGroovy", "aRingToneDigitalRain", "aRingToneMagical", "aRingToneDejaVu", "aRingToneOfficePhone"];
        
        var sFileName: String = "";
        
        if(nRingToneId < 0 || nRingToneId >= aFileName.count)
        {
            m_nRingToneId = cVaxUserAgentInfo.VAX_RING_TONE_DIGITAL_RAIN;
            sFileName = "aRingToneDigitalRain";
        }
        else
        {
            m_nRingToneId = nRingToneId;
            sFileName = aFileName[nRingToneId];
        }
        
        if (mRingToneDATA != nil)
        {
            mRingToneDATA = nil
        }
        
        mRingToneDATA = NSMutableData(contentsOfFile: Bundle.main.path(forResource: sFileName, ofType: "mp3")!)
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////
    
    func GetRingTone() ->Int
    {
        return m_nRingToneId
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////
    
    func StartRingTone() ->Void
    {
        if (mRingTonePLAY != nil)
        {
            return
        }
        
        mRingTonePLAY = cAVAudioPlayer()
        
        mRingTonePLAY?.PlayLoadWithData(mRingToneDATA!)
        mRingTonePLAY?.PlayStart(-1)
    }
    
    func StopRingTone()
    {
        if (mRingTonePLAY != nil)
        {
            mRingTonePLAY?.PlayStop()
            mRingTonePLAY = nil
        }
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func OnInitialized() ->Void
    {
        self.BusyRing(true);
        self.DialRing(true);
        
        cSettingRingToneTableView.PostInitialize()
    }
    
    func OnRingToneStarted(_ sAccountName: String!) ->Void
    {
        // For devices, sample code utilizes Call-Kit framework.
        
        #if targetEnvironment(simulator)
            self.StartRingTone()
        #endif
    }
        
    func OnRingToneEnded(_ sAccountName: String!) ->Void
    {
        // For devices, sample code utilizes Call-Kit framework.
        
        #if targetEnvironment(simulator)
            self.StopRingTone()
        #endif
    }
}
