
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

@objc class cVaxPhoneChat : NSObject, iVaxUserAgentSIP
{
    var mVaxAgentSIP: cVaxUserAgentSIP!
    static var mSelf: cVaxPhoneChat?
         
    override init()
    {
        super.init()
        
        cVaxPhoneChat.mSelf = self
        mVaxAgentSIP = cVaxUserAgentSIP(iDelegate: self)
    }
    
    deinit
    {
        mVaxAgentSIP = nil
    }
    
    static func GetSelf() ->cVaxPhoneChat
    {
        return mSelf!
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func ChatSubscribeContactAll() ->Void
    {
        mVaxAgentSIP.ChatSubscribeContactAll(cVaxPhoneSIP.ACCOUNT_VAXVOIP);
    }
    
    func ChatSetMyStatus(_ nStatusId: Int) ->Bool
    {
        return mVaxAgentSIP.ChatSetMyStatus(cVaxPhoneSIP.ACCOUNT_VAXVOIP, nStatusId)
    }
    
    func LoadChatContactAll() ->Void
    {
        let vStoreDB: cStoreChatContacts = cStoreChatContacts()
        let aContactAll: [Any]  = vStoreDB.GetContactAll()
        
        for nRecordNo in 0..<aContactAll.count
        {
            let vDataDB: cStoreDataChatContacts = aContactAll[nRecordNo] as! cStoreDataChatContacts
            
            mVaxAgentSIP.ChatAddContact(cVaxPhoneSIP.ACCOUNT_VAXVOIP, vDataDB.m_sName, vDataDB.m_bPresence)
            cChatContactTableView.PostChatContactAdded(sContactName: vDataDB.m_sName, nMissedCount: vDataDB.m_nMissedCount, bPresence: vDataDB.m_bPresence)
        }
    }
    
    func ChatAddContact(_ sUserName: String, _ bPresence: Bool) ->Bool
    {
        if !mVaxAgentSIP.ChatAddContact(cVaxPhoneSIP.ACCOUNT_VAXVOIP, sUserName, bPresence)
        {
            return false
        }
        
        let vStoreDB: cStoreChatContacts = cStoreChatContacts()
        vStoreDB.AddContact(sUserName, 0, bPresence)
        
        cChatContactTableView.PostChatContactAdded(sContactName: sUserName, nMissedCount: 0, bPresence: bPresence)
        return true
    }
    
    func ChatRemoveContact(_ sUserName: String) ->Bool
    {
        if !(mVaxAgentSIP.ChatRemoveContact(cVaxPhoneSIP.ACCOUNT_VAXVOIP, sUserName))
        {
            return false
        }
        
        let vChatContactDB: cStoreChatContacts = cStoreChatContacts()
        vChatContactDB.RemoveContact(sUserName)
        
        let vChatMsgDB: cStoreChatMsgs = cStoreChatMsgs()
        vChatMsgDB.RemoveMessageAll(sUserName)
        
        return true
    }
    
    func ChatSendMessageText(_ sUserName: String, _ sMsgText: String) ->Bool
    {
        if !mVaxAgentSIP.ChatSendMessageText(cVaxPhoneSIP.ACCOUNT_VAXVOIP, sUserName, sMsgText, false, 0)
        {
            return false
        }
        
        let vStoreDB: cStoreChatMsgs = cStoreChatMsgs()
        vStoreDB.AddChatMessageText(sUserName, sMsgText, true)
        
        cChatContactTableView.PostChatMessageText(sContactName: sUserName, bIsChatContact: mVaxAgentSIP.ChatFindContact(cVaxPhoneSIP.ACCOUNT_VAXVOIP, sUserName), sMsgText: sMsgText, bMsgTypeOutgoing: true)
        
        return true
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func OnInitialized() ->Void
    {
        cChatContactTableView.PostInitialize()
    }
    
    func OnChatContactStatus(_ sAccountName: String!, _ sUserName :String!, _ nStatusId :Int) ->Void
    {
        cChatContactTableView.PostChatContactStatus(sContactName: sUserName, nStatusId: nStatusId)
    }
            
    func OnChatRecvMsgText(_ sAccountName: String!, _ sUserName :String!, _ sMsgText :String!, _ bIsChatContact :Bool) ->Void
    {
        if !bIsChatContact
        {
            self.ChatAddContact(sUserName, false)
        }
        
        let vStoreDB: cStoreChatMsgs = cStoreChatMsgs()
        vStoreDB.AddChatMessageText(sUserName, sMsgText, false)
        
        cChatContactTableView.PostChatMessageText(sContactName: sUserName, bIsChatContact: bIsChatContact, sMsgText: sMsgText, bMsgTypeOutgoing: false)
        AppDelegate.PostChatMsgRecvText()
    }
}
