
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

@objc class cVaxPhoneGeneral: NSObject, iVaxUserAgentSIP
{
    var mVaxAgentSIP: cVaxUserAgentSIP!
    static var mSelf: cVaxPhoneGeneral?
    
    override init()
    {
        super.init()
        
        cVaxPhoneGeneral.mSelf = self
        mVaxAgentSIP = cVaxUserAgentSIP(iDelegate: self)
    }
    
    deinit
    {
        mVaxAgentSIP = nil
    }
    
    static func GetSelf() ->cVaxPhoneGeneral
    {
        return mSelf!
    }

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func SpeakerPhone(_ bEnable: Bool) ->Bool
    {
        return mVaxAgentSIP.SpeakerPhone(bEnable)
    }
    
    func IsSpeakerPhone() ->Bool
    {
        return mVaxAgentSIP.IsSpeakerPhone()
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func IsLineBusy() ->Bool
    {
        return mVaxAgentSIP.IsLineBusy(0)
    }
    
    func IsLineConnected() ->Bool
    {
        return mVaxAgentSIP.IsLineConnected(0)
    }
    
    func IsLineHold() ->Bool
    {
        return mVaxAgentSIP.IsLineHold(0)
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    
    func HoldLine() ->Bool
    {
        return mVaxAgentSIP.HoldLine(0)
    }
    
    func UnHoldLine() ->Bool
    {
        return mVaxAgentSIP.UnHoldLine(0)
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    
    func IsNetworkAvailable() ->Bool
    {
        return mVaxAgentSIP.IsNetworkAvailable()
    }
    
    func NetworkReachability(_ bEnable: Bool) ->Bool
    {
        return mVaxAgentSIP.NetworkReachability(bEnable)
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    
    func OnInitialized() ->Void
    {
        cVideoView.PostInitialize()
        cSettingsTableView.PostInitialize()
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    
    func OnDialCallStarted(_ sAccountName: String!, _ nLineNo :Int, _ sCallerName :String!, _ sCallerId :String!, _ sDialNo :String!) ->Void
    {
        cVideoView.PostDialCallStarted()
    }
    
    func OnDialCallFailed(_ sAccountName: String!, _ nLineNo :Int, _ nStatusCode :Int, _ sReasonPhrase :String!, _ sContact :String!) ->Void
    {
        cVideoView.PostDialCallFailed(nStatusCode: nStatusCode, sReasonPhrase: sReasonPhrase)
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    
    func OnConnectedCall(_ sAccountName: String!, _ nLineNo: Int) ->Void
    {
        UIDevice.current.isProximityMonitoringEnabled = true
        cVideoView.PostConnectedCall()
    }
    
    func OnHungupCall(_ sAccountName: String!, _ nLineNo: Int) ->Void
    {
        UIDevice.current.isProximityMonitoringEnabled = false
        cVideoView.PostDisconnectedCall()
    }
    
    func OnDisconnectedCall(_ nLineNo :Int) ->Void
    {
        UIDevice.current.isProximityMonitoringEnabled = false
        cVideoView.PostDisconnectedCall()
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    
    func OnAddCallHistory(_ sAccountName: String!, _ bOutboundCallType :Bool, _ sCallerName :String!, _ sCallerId :String!, _ sDialNo :String!, _ nStartTime :Int, _ nEndTime :Int, _ nDuration :Int, _ nDayNo :Int, _ nHistoryTypeId :Int) ->Void
    {
        var vResultName: String = ""
        var vResultPhoneNo: String = ""
        var vResultContactId: String = ""
        
        if bOutboundCallType
        {
            cVaxCallInfo.PrepareInfo(sCallerName, sDialNo, &vResultName, &vResultPhoneNo, &vResultContactId)
        }
        else
        {
            AppDelegate.PostInboundCallHistory()
            cVaxCallInfo.PrepareInfo(sCallerName, sCallerId, &vResultName, &vResultPhoneNo, &vResultContactId)
        }
        
        let vStoreDB: cStoreRecents = cStoreRecents()
        let nRecordId: Int = vStoreDB.AddCallHistory(bOutboundCallType, vResultName, vResultPhoneNo, nStartTime, nEndTime, nDuration, nDayNo, nHistoryTypeId)
        
        cRecentTableView.PostAddCallHistory(nRecordId, bOutboundCallType, vResultName, vResultPhoneNo, nStartTime, nEndTime, nDuration, nDayNo, nHistoryTypeId)
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////
}
