
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

@objc class cVaxPhoneVideo: NSObject, iVaxUserAgentSIP
{
    var mVaxAgentSIP: cVaxUserAgentSIP!
    static var mSelf: cVaxPhoneVideo?
    
    var m_bFrontVideoCameraCaptured: Bool

    var m_bVideoEnabled: Bool
    var m_bVideoDeviceCaptured: Bool
                
    override init()
    {
        m_bVideoEnabled = true
        m_bVideoDeviceCaptured = false
        m_bFrontVideoCameraCaptured = true
        
        super.init()
        
        cVaxPhoneVideo.mSelf = self
       
        mVaxAgentSIP = cVaxUserAgentSIP(iDelegate: self)
    }
    
    deinit
    {
        mVaxAgentSIP = nil
    }
    
    static func GetSelf() ->cVaxPhoneVideo
    {
        return mSelf!
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    
    func IsVideoEnabled() ->Bool
    {
        return m_bVideoEnabled
    }
    
    func EnableVideo(_ bEnable: Bool) ->Bool
    {
        if !mVaxAgentSIP.EnableVideo(0, bEnable, bEnable)
        {
            return false
        }
        
        m_bVideoEnabled = bEnable
        return true
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    
    func ApplyVideoQuality() ->Void
    {
        if !m_bVideoDeviceCaptured
        {
            return
        }
        
        self.CloseVideoDevice()
        self.OpenVideoDevice()
    }
    
    func OpenVideoDevice() ->Bool
    {
        if m_bVideoDeviceCaptured
        {
            return true
        }
        
        self.CloseVideoDevice()
        
        let nQuality: Int = cSettingVideoQualityTableView.GetVideoQuality()
        
        var nDeviceId: Int = 0
        
        if m_bFrontVideoCameraCaptured
        {
            nDeviceId = 0
        }
        else
        {
            nDeviceId = 1
        }
        
        if !mVaxAgentSIP.OpenVideoDev(nDeviceId, nQuality)
        {
            return false
        }
        
        m_bVideoDeviceCaptured = true
        return true
    }
    
    func CloseVideoDevice() ->Void
    {
        m_bVideoDeviceCaptured = false
        mVaxAgentSIP.CloseVideoDev()
    }
    
    func RotateVideoDevice() ->Void
    {
        if !m_bVideoDeviceCaptured
        {
            return
        }
        
        if m_bFrontVideoCameraCaptured
        {
            m_bFrontVideoCameraCaptured = false
        }
        else
        {
            m_bFrontVideoCameraCaptured = true
        }
        
        self.CloseVideoDevice()
        self.OpenVideoDevice()
    }
    
    func ApplyVideoBitrate(_ nQuality: Int) ->Void
    {
        mVaxAgentSIP.VideoCodecBitRate(cVaxUserAgentInfo.VAX_VIDEO_CODEC_VP8, nQuality)
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func OnInitialized() ->Void
    {
        cSettingVideoBitrateTableView.PostInitialize()
    }
    
    func OnUnInitialized() ->Void
    {
        CloseVideoDevice()
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func OnVideoDeviceFrameRGB(_ nDeviceId :Int, _ pFrameRGB :UnsafeMutablePointer<Int8>!, _ nFrameSize :Int, _ nFrameWidth :Int, _ nFrameHeight :Int) ->Void
    {
        cVideoView.PostDeviceFrameRGB(pFrameRGB, nFrameSize, nFrameWidth, nFrameHeight)
    }
    
    func OnVideoRemoteFrameRGB(_ sAccountName: String!, _ nLineNo :Int, _ pFrameRGB :UnsafeMutablePointer<Int8>!, _ nFrameSize :Int, _ nFrameWidth :Int, _ nFrameHeight :Int) ->Void
    {
        cVideoView.PostRemoteFrameRGB(pFrameRGB, Int(nFrameSize), Int(nFrameWidth), Int(nFrameHeight))
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
}
