
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

@objc class cVaxPhoneSIP: NSObject, iVaxUserAgentSIP, iVaxCallKit, iIncomingCallAlert, iVaxBackForeNotify
{
    static var ACCOUNT_VAXVOIP: String = "VaxAccountSIP"
    static let ACCOUNT_REGISTER_EXPIRE = 1800
    
    static var PUSH_TEAM_ID: String = "B4NJYW6YVA"
    static var PUSH_BUNDLE_ID: String = "com.VaxSoft.VaxPhoneSDK.iOS"
    
    static var mSelf: cVaxPhoneSIP?
    
    var m_bOnline: Bool = false
    var m_bRegistrationSIP: Bool = false
    
    var mVaxAgentSIP: cVaxUserAgentSIP!
    
    var mIncomingCallAlert: cIncomingCallAlert!
    var mBackForeNotify: cVaxBackForeNotify!
    
    var mVaxPhoneGeneral: cVaxPhoneGeneral!
    var mVaxPhoneCodec: cVaxPhoneCodec!
    var mVaxPhoneRing: cVaxPhoneRing!
    var mVaxPhoneDTMF: cVaxPhoneDTMF!
    var mVaxPhoneMicSpk: cVaxPhoneMicSpk!
    var mVaxPhoneVideo: cVaxPhoneVideo!
    var mVaxPhoneStatus: cVaxPhoneStatus!
    var mVaxPhoneChat: cVaxPhoneChat!
    var mVaxPhoneLog: cVaxPhoneLog!
    var mVaxPhoneVoiceChanger: cVaxPhoneVoiceChanger!
    
    var mCallKit: cVaxCallKit!
                
    override init()
    {
        m_bOnline = false;
        m_bRegistrationSIP = false
        
        super.init()
        cVaxPhoneSIP.mSelf = self
       
        mIncomingCallAlert = cIncomingCallAlert(iDelegate: self)
        mVaxAgentSIP = cVaxUserAgentSIP(iDelegate: self)
        
        mVaxPhoneGeneral = cVaxPhoneGeneral()
        mVaxPhoneCodec = cVaxPhoneCodec()
        
        mVaxPhoneRing = cVaxPhoneRing()
        mVaxPhoneDTMF = cVaxPhoneDTMF()
        
        mVaxPhoneMicSpk = cVaxPhoneMicSpk()
        mVaxPhoneVideo = cVaxPhoneVideo()
        
        mVaxPhoneStatus = cVaxPhoneStatus()
        mVaxPhoneChat = cVaxPhoneChat()
        
        mVaxPhoneLog = cVaxPhoneLog();
        mVaxPhoneVoiceChanger = cVaxPhoneVoiceChanger()
        
        mCallKit = cVaxCallKit(iDelegate: self)
        mBackForeNotify = cVaxBackForeNotify(iDelegate: self)
    }
    
    deinit
    {
        mIncomingCallAlert = nil
    }
    
    static func GetSelf() ->cVaxPhoneSIP
    {        
        return mSelf!
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func Initialize() ->Bool
    {
        let vStore: cStoreAccount = cStoreAccount()
        
        let bRegistrationSIP: Bool = vStore.GetRegistration()
        
        let sUserName: String = vStore.GetUserName()
        let sDisplayName: String = vStore.GetDisplayName()
        
        let sAuthLogin: String = vStore.GetAuthLogin()
        let sAuthPwd: String = vStore.GetAuthPwd()
        
        let sDomainRealm: String = vStore.GetDomainRealm()
        
        let sServerAddr: String = vStore.GetServerAddr()
        let sServerPort: String = vStore.GetServerPort()
        
        ///////////////////////////////////////////////////////////////////////////////
        
        if (sAuthLogin.count) == 0
        {
            cMsgAlert.ShowAlert("Please enter Auth Login")
            return false
        }
        
        if (sServerAddr.count) == 0
        {
            cMsgAlert.ShowAlert("Please enter SIP Server address")
            return false
        }
        
        ///////////////////////////////////////////////////////////////////////////////
        
        mVaxAgentSIP.SetLicenceKey("TRIAL-LICENSE-KEY")
        
        ///////////////////////////////////////////////////////////////////////////////
        
        var nLocalPortSIP: Int = -1
        
        if !(cSettingNetworkTableView.IsRandomPortSIP())
        {
            nLocalPortSIP = cSettingNetworkTableView.GetNetworkPortSIP()
        }
        
        if !mVaxAgentSIP.Initialize()
        {
            return false
        }
        
        //mVaxAgentSIP.SetNetworkPrioritySIP(cVaxUserAgentInfo.VAX_NETWORK_MODE_PRIORITY_IPV4)
        
        mVaxAgentSIP.RemoveAccountSIP(cVaxPhoneSIP.ACCOUNT_VAXVOIP);
        
        if !mVaxAgentSIP.AddAccountSIP(cVaxPhoneSIP.ACCOUNT_VAXVOIP, cVaxUserAgentInfo.VAX_ACCOUNT_TYPE_UDP, "", nLocalPortSIP, sDisplayName, sUserName, sAuthLogin, sAuthPwd, sDomainRealm, sServerAddr, Int(sServerPort)!)
        {
            return false
        }
        
        m_bRegistrationSIP = bRegistrationSIP
        
        if bRegistrationSIP
        {
            if !mVaxAgentSIP.RegisterAccountPushKit(cVaxPhoneSIP.ACCOUNT_VAXVOIP, cVaxPhoneSIP.ACCOUNT_REGISTER_EXPIRE)
            {
                return false
            }
        }
        
        mVaxAgentSIP.AutoRegistration(cVaxPhoneSIP.ACCOUNT_VAXVOIP, true, -1, 20)
        mVaxAgentSIP.EnableKeepAlive(cVaxPhoneSIP.ACCOUNT_VAXVOIP, 10) // 10 seconds keep alive;
        
        mBackForeNotify.BackForegroundNotify(true)
        mVaxAgentSIP.NetworkReachability(true)
        
        m_bOnline = true
        return true
    }
    
    func UnInitialize() ->Void
    {
        m_bOnline = false
        m_bRegistrationSIP = false
        
        mVaxAgentSIP.NetworkReachability(false)
        mBackForeNotify.BackForegroundNotify(false)
        
        mVaxAgentSIP.UnRegisterAccountSIP(cVaxPhoneSIP.ACCOUNT_VAXVOIP)
    }
    
    func IsOnline() ->Bool
    {
        return m_bOnline;
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
        
    func DialCall(_ sDialNo: String) ->Bool
    {
        return mVaxAgentSIP.DialCall(cVaxPhoneSIP.ACCOUNT_VAXVOIP, 0, "", "", sDialNo)
    }
    
    func DisconnectCall() ->Bool
    {
        return mVaxAgentSIP.DisconnectCall(0)
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////
    
    func OnSuccessToRegister(_ sAccountName: String!) ->Void
    {
        cAccountLoginView.PostSuccessToRegister()
    }
    
    func OnVaxRegisterSuccessPushKit(_ sDeviceToken: String) ->Void
    {
        if !m_bRegistrationSIP
        {
            return;
        }
        
        mVaxAgentSIP.SetPushSIP(cVaxPhoneSIP.ACCOUNT_VAXVOIP, cVaxPhoneSIP.PUSH_TEAM_ID, cVaxPhoneSIP.PUSH_BUNDLE_ID, sDeviceToken);
        
        mVaxAgentSIP.RegisterAccountSIP(cVaxPhoneSIP.ACCOUNT_VAXVOIP, cVaxPhoneSIP.ACCOUNT_REGISTER_EXPIRE);
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////
    
    func OnIncomingCallPushKit(_ sCallerName :String!, _ sCallerId :String!)  ->Void
    {
        if !m_bRegistrationSIP || mVaxAgentSIP.IsLineBusy(0)
        {
            return;
        }
    
        mVaxAgentSIP.RegisterAccountPushKit(cVaxPhoneSIP.ACCOUNT_VAXVOIP, cVaxPhoneSIP.ACCOUNT_REGISTER_EXPIRE);
        
        mCallKit.ShowUI(sCallerName, sCallerId)
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////
    
    func OnIncomingCallStarted(_ sAccountName: String!, _ sCallerName :String!, _ sCallerId :String!) ->Void
    {
        if mVaxAgentSIP.IsLineConnected(0)
        {
            mVaxAgentSIP.RejectCall();
            return;
        }
        
        #if targetEnvironment(simulator)
            mIncomingCallAlert.ShowAlert(sCallerName, sCallerId)
        #else
            mCallKit.ShowUI(sCallerName, sCallerId)
        #endif
    }
    
    func OnIncomingCallEnded(_ sAccountName: String!, _ nReasonCode: Int) ->Void
    {
        #if targetEnvironment(simulator)
            mIncomingCallAlert.HideAlert()
        #else
        if nReasonCode != cVaxUserAgentInfo.VAX_INCOMING_CALL_ACCEPTED
        {
            mCallKit.HideUI()
        }
        #endif
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////

    func OnVaxAnswerCallKit() ->Void
    {
        mVaxAgentSIP.AcceptCall(0)
    }
    
    func OnVaxEndCallKit() ->Void
    {
        if(mVaxAgentSIP.IsLineConnected(0))
        {
            mVaxAgentSIP.DisconnectCall(0)
        }
        else
        {
            mVaxAgentSIP.RejectCall()
        }
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////
    
    func OnHungupCall(_ sAccountName: String!, _ nLineNo: Int) ->Void
    {
        mCallKit.HideUI()
    }
    
    func OnDisconnectedCall(_ nLineNo :Int) ->Void
    {
        mCallKit.HideUI()
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////
    
    func OnIncomingCallAlertAccept() ->Void
    {
        mVaxAgentSIP.AcceptCall(0)
    }
    
    func OnIncomingCallAlertReject() ->Void
    {
        mVaxAgentSIP.RejectCall()
    }
    
    func OnIncomingCallAlertMute() ->Void
    {
        mVaxPhoneRing.MuteRingTone()
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    
    func OnNetworkReachability(_ bAvailable :Bool) ->Void
    {
        if (bAvailable == false) || (self.IsOnline() == false)
        {
            return;
        }
        
        if mVaxAgentSIP.IsLineConnected(0)
        {
            mVaxAgentSIP.RefreshLine(0);
        }
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func OnVaxApplicationEnterForeground() ->Void
    {
        if m_bRegistrationSIP
        {
            mVaxAgentSIP.RegisterAccountPushKit(cVaxPhoneSIP.ACCOUNT_VAXVOIP, cVaxPhoneSIP.ACCOUNT_REGISTER_EXPIRE);
        }
    }
    
    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
}
