
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

var mvAccountLoginTableView: cAccountLoginView! = nil
var mvAccountLoginStatusText: String! = nil

class cAccountLoginView: UIViewController
{
    let DEFAULT_SIP_PORT  = "5060"
    
    @IBOutlet var mAuthLogin: cCustomTextField!
    @IBOutlet var mAuthPassword: cCustomTextField!
    
    @IBOutlet var mServerURI: cCustomTextField!
    @IBOutlet var mRegisterSIP: UISwitch!
    
    @IBOutlet var mStatusText: UILabel!
    @IBOutlet var mOnLineBtn: UIButton!
    
    @IBOutlet var mScrollView: UIScrollView!
    
    var m_bCheckAccountStatus: Bool  = true
    
    override func viewDidLoad()
    {
        m_bCheckAccountStatus = true
        
        super.viewDidLoad()

        mAuthLogin.EnableKeyboardReturnKey()
        mAuthPassword.EnableKeyboardReturnKey()
        mServerURI.EnableKeyboardReturnKey()
        
        mvAccountLoginStatusText = ""
        mvAccountLoginStatusText = "Account is offline"
        
        mScrollView.contentSize = mScrollView.frame.size
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    override func viewWillAppear(_ animated: Bool)
    {
        mvAccountLoginTableView = self
        
        self.DisplayLoginInfo()
       
        self.ShowStatusMsg(sMsgtext: mvAccountLoginStatusText)
        self.UpdateOnlineBtnText()
        
        if m_bCheckAccountStatus
        {
            m_bCheckAccountStatus = false;
            let vStore: cStoreAccount = cStoreAccount()
        
            if vStore.GetAccountStatusOnline()
            {
                self.AccountOnline()
            }
        }
        
        super.viewWillAppear(animated)
    }
    
    override func viewWillDisappear(_ animated: Bool)
    {
        mvAccountLoginTableView = nil
        super.viewWillDisappear(animated)
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    static func PostStatusText(_ sMsgText: String) ->Void
    {
        if mvAccountLoginStatusText != nil
        {
            mvAccountLoginStatusText = sMsgText
        }
        
        if mvAccountLoginTableView != nil
        {
            mvAccountLoginTableView.ShowStatusMsg(sMsgtext: sMsgText)
        }
    }
    
    static func PostSuccessToRegister() ->Void
    {
        if mvAccountLoginTableView != nil
        {
            let vStore: cStoreAccount = cStoreAccount()
            vStore.SetAccountStatusOnline(true)
            
            mvAccountLoginTableView.ShowMainTabBar(bStart: true)
        }
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    @IBAction func OnClickSignin(_ sender: Any)
    {
        if cVaxPhoneSIP.GetSelf().IsOnline()
        {
            self.AccountOffline()
            return
        }
        
        self.AccountOnline()
    }
    
    func AccountOffline() ->Void
    {
        self.ShowMainTabBar(bStart: false)
        
        cVaxPhoneSIP.GetSelf().UnInitialize()
        self.UpdateOnlineBtnText()
        
        let vStore: cStoreAccount = cStoreAccount()
        vStore.SetAccountStatusOnline(false)
    }
    
    func AccountOnline() ->Void
    {
        if !cVaxPhoneGeneral.GetSelf().IsNetworkAvailable()
        {
            cAccountLoginView.PostStatusText("Network is not available")
            return;
        }
        
        self.StoreLoginInfo()
        
        let bRegisterSIP = mRegisterSIP.isOn
        
        if !cVaxPhoneSIP.GetSelf().Initialize()
        {
            return
        }
        
        self.UpdateOnlineBtnText()
        
        if bRegisterSIP == false
        {
            let vStore: cStoreAccount = cStoreAccount()
            vStore.SetAccountStatusOnline(true)
            
            self.ShowMainTabBar(bStart: true)
        }
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    @IBAction func OnClickTestAccount(_ sender: Any)
    {
        let nDateTime: TimeInterval = Date().timeIntervalSince1970
        let sDateTime = String(format: "%ld", Int(nDateTime) % 9999999)
        
        mAuthLogin.text = sDateTime as String
        mAuthPassword.text = sDateTime as String
        
        mServerURI.text = "demo.vaxvoip.com:8060"
        mRegisterSIP.isOn = true
        
        let vStore: cStoreAccount = cStoreAccount()
        
        vStore.SetUserName(sDateTime as String)
        vStore.SetDisplayName(sDateTime as String)
        vStore.SetDomainRealm("demo.vaxvoip.com")
        
        self.ShowTestAccountInfo(mAuthLogin.text!, mAuthPassword.text!, "demo.vaxvoip.com", "8060")
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    func ShowStatusMsg(sMsgtext: String) ->Void
    {
        mStatusText.text = sMsgtext
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    @IBAction func OnEditingLoginAuth(_ sender: Any)
    {
        let vStore: cStoreAccount = cStoreAccount()
        
        vStore.SetUserName("")
        vStore.SetDisplayName("")
        vStore.SetDomainRealm("")
    }
    
    @IBAction func OnEditingServerAddr(_ sender: Any)
    {
        let vStore: cStoreAccount = cStoreAccount()
        
        vStore.SetUserName("")
        vStore.SetDisplayName("")
        vStore.SetDomainRealm("")
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    override var prefersStatusBarHidden: Bool
    {
        get
        {
            return false
        }
    }
    
    @objc func OnShowMainTabBar() ->Void
    {
        self.performSegue(withIdentifier: "SegueMainTabBar", sender: self)
    }
    
    func ShowMainTabBar(bStart: Bool) ->Void
    {
        NSObject.cancelPreviousPerformRequests(withTarget: self, selector: #selector(self.OnShowMainTabBar), object: nil)
        
        if bStart == true
        {
            perform(#selector(self.OnShowMainTabBar), with: nil, afterDelay: 1.0)
        }
    }

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    func UpdateOnlineBtnText()
    {
        if cVaxPhoneSIP.GetSelf().IsOnline()
        {
            mOnLineBtn.setTitle("Sign out", for: .normal)
        }
        else
        {
            mOnLineBtn.setTitle("Sign in", for: .normal)
        }
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    @IBAction func UnwindFromAccountInfoTableView(_ segue:UIStoryboardSegue)
    {
        
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?)
    {
        if (segue.identifier == "SegueAccountInfo")
        {
            let vStore: cStoreAccount? = cStoreAccount()
            
            var sUserName = vStore?.GetUserName()
            if sUserName == nil || (sUserName?.count == 0)
            {
               sUserName = mAuthLogin.text
            }
            
            var sDisplayName = vStore?.GetDisplayName()
            if sDisplayName == nil || sDisplayName?.count == 0
            {
                sDisplayName = mAuthLogin.text
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////
            
            var sServerAddr: String = ""
            var sServerPort: String = ""
            
            self.ParseServerURI(sURI: mServerURI.text, sResultServerAddr: &sServerAddr, sResultServerPort: &sServerPort)
            
            //////////////////////////////////////////////////////////////////////////////////////////////////
            
            var sDomainRealm: String? = vStore?.GetDomainRealm()
            if sDomainRealm == nil || sDomainRealm?.count == 0
            {
                sDomainRealm = sServerAddr
            }
            
            cAccountInfoTableView.PreloadAccountInfo(sUserName: sUserName!, sDisplayName: sDisplayName!, sAuthLogin: mAuthLogin.text!, sAuthPwd: mAuthPassword.text!, sDomainRealm: sDomainRealm!, sServerAddr: sServerAddr, sServerPort: sServerPort, bRegisterSIP: mRegisterSIP.isOn)
        }
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    func ShowTestAccountInfo(_ sAuthLogin: String, _ sAuthPwd: String, _ sServerAddr: String, _ sServerPort: String)
    {
        let sMsgText = String(format:"Auth Login: %@\nAuth Password: %@\nServer Addr: %@\nServer Port: %@\n\nDemo accounts are only for account-to-account VoIP calls. Use demo accounts in different devices to dial/receive VoIP calls.\n\nDial 000 for self testing.\n\n[Powered by VaxVoIP Server SDK]", sAuthLogin, sAuthPwd, sServerAddr, sServerPort);
        
        let MsgAlert = UIAlertController(title: "VaxPhone", message: sMsgText, preferredStyle: .alert)
        
        let OkAction = UIAlertAction(title: "OK", style: .default, handler: {(_ action: UIAlertAction) -> Void in
        })
        
        MsgAlert.addAction(OkAction)
        
        self.present(MsgAlert, animated: true, completion: nil)
    }
    
/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

    func GenServerURI() ->String
    {
        let vStore: cStoreAccount = cStoreAccount()
        let sServerAddr: String = vStore.GetServerAddr()
        
        if sServerAddr.count == 0
        {
            return ""
        }
        
        let sServerPort: String = vStore.GetServerPort()
        
        let rangeValue: NSRange = (sServerAddr as NSString).range(of: ":", options: .caseInsensitive)
        
        if rangeValue.length > 0
        {
            //Server Address is IPv6
            
            return String(format: "[%@]:%@", sServerAddr, sServerPort)
        }
        
        return String(format: "%@:%@", sServerAddr, sServerPort)
    }
    
    func DisplayLoginInfo() ->Void
    {
        let vStore: cStoreAccount = cStoreAccount()
        
        mAuthLogin.text = vStore.GetAuthLogin()
        mAuthPassword.text = vStore.GetAuthPwd()
        
        mServerURI.text = self.GenServerURI()
        mRegisterSIP.isOn = vStore.GetRegistration()
    }
    
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
    
    func ParseServerURI(sURI: String?, sResultServerAddr: inout String, sResultServerPort: inout String) ->Void
    {
        if sURI == nil || sURI?.count == 0
        {
            return
        }
        
        var aCompURI: [Any] = sURI!.components(separatedBy: ":")
        
        if aCompURI.count == 1
        {
            //URI is IPv4 without Port
            
            sResultServerAddr = aCompURI[0] as! String
            sResultServerPort = DEFAULT_SIP_PORT
            
            return
        }
        
        if aCompURI.count == 2
        {
            //URI is IPv4 with Port
            
            sResultServerAddr = aCompURI[0] as! String
            sResultServerPort = aCompURI[1] as! String
            
            return
        }
        
        aCompURI = sURI!.components(separatedBy: "]:")
        
        if aCompURI.count == 1
        {
            // URI is IPv6 without port
            
            var sServerAddr: String = aCompURI[0] as! String
            
            sServerAddr = sServerAddr.replacingOccurrences(of: "[", with: "")
            sServerAddr = sServerAddr.replacingOccurrences(of: "]", with: "")
            
            sResultServerAddr = sServerAddr
            sResultServerPort = DEFAULT_SIP_PORT
            
            return
        }
        
        var sServerAddr: String = aCompURI[0] as! String
        
        sServerAddr = sServerAddr.replacingOccurrences(of: "[", with: "")
        
        sResultServerAddr = sServerAddr
        sResultServerPort = aCompURI[1] as! String
    }
    
    func StoreLoginInfo() ->Void
    {
        let sAuthLogin: String = mAuthLogin.text!
        let sAuthPwd: String = mAuthPassword.text!
        
        let sServerURI: String = mServerURI.text!
        let bRegister: Bool = mRegisterSIP.isOn
        
        ///////////////////////////////////////////////////////////////////////////////
        
        var sServerAddr: String = ""
        var sServerPort: String = ""
        
        self.ParseServerURI(sURI: sServerURI, sResultServerAddr: &sServerAddr, sResultServerPort: &sServerPort)
        
        ///////////////////////////////////////////////////////////////////////////////
        
        let vStore: cStoreAccount = cStoreAccount()
        
        vStore.SetAuthLogin(sAuthLogin)
        vStore.SetAuthPwd(sAuthPwd)
        
        vStore.SetServerAddr(sServerAddr)
        vStore.SetServerPort(sServerPort)
        
        vStore.SetRegistration(bRegister)
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
}
