
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

protocol iIncomingCallAlert: NSObjectProtocol
{
    func OnIncomingCallAlertAccept() ->Void
    func OnIncomingCallAlertReject() ->Void
    func OnIncomingCallAlertMute() ->Void
}

class cIncomingCallAlert: NSObject
{
    var mIncomingCallAlert: UIAlertController?
    var mCallAlertDelegate: iIncomingCallAlert!
    
    init(iDelegate: Any)
    {
        mCallAlertDelegate = (iDelegate as! iIncomingCallAlert)
        super.init()
    }
    
    deinit
    {
        self.HideAlert()
    }
    
    private func PrepareTitle(_ sCallerName: String, _ sCallerId: String) ->String
    {
        if sCallerName.compare(sCallerId, options: .caseInsensitive) != .orderedSame
        {
            return sCallerId
        }
        return NSString(format: "%@-%@", sCallerName, sCallerId) as String
    }
    
    func ShowAlert(_ sCallerName: String, _ sCallerId: String) ->Void
    {
        self.HideAlert()
        
        let sTitle = self.PrepareTitle(sCallerName, sCallerId)
        
        if UI_USER_INTERFACE_IDIOM() == .pad
        {
            // if user's device is iPad.
            mIncomingCallAlert = UIAlertController(title: sTitle, message: "", preferredStyle: .alert)
        }
        else
        {
            mIncomingCallAlert = UIAlertController(title: sTitle, message: "", preferredStyle: .actionSheet)
        }
        
        let RejectCallAction = UIAlertAction(title: "Reject Call", style: .default, handler: {(_ action: UIAlertAction) -> Void in
            self.mCallAlertDelegate.OnIncomingCallAlertReject()
            self.HideAlert()
        })
        
        let AcceptCallAction = UIAlertAction(title: "Accept Call", style: .default, handler: {(_ action: UIAlertAction) -> Void in
            self.mCallAlertDelegate.OnIncomingCallAlertAccept()
            cMainTabBarController.SwitchToVideoTab(sPhoneNo: "")
            self.HideAlert()
        })
        let MuteRingAction = UIAlertAction(title: "Mute Ring", style: .default, handler: {(_ action: UIAlertAction) -> Void in
            self.mCallAlertDelegate.OnIncomingCallAlertMute()
            self.HideAlert()
        })
        let CancelAction = UIAlertAction(title: "Cancel", style: .default, handler: {(_ action: UIAlertAction) -> Void in
            self.HideAlert()
        })
        
        RejectCallAction.setValue(UIColor.red, forKey: "titleTextColor")
        
        mIncomingCallAlert?.addAction(RejectCallAction)
        mIncomingCallAlert?.addAction(AcceptCallAction)
        mIncomingCallAlert?.addAction(MuteRingAction)
        mIncomingCallAlert?.addAction(CancelAction)
        
        var vMainTabBar: UIViewController? = UIViewController()
        vMainTabBar = cMainTabBarController.GetMainTabBar()
        
        let popoverPresentationController: UIPopoverPresentationController? = mIncomingCallAlert?.popoverPresentationController
        
        if popoverPresentationController != nil
        {
            mIncomingCallAlert?.popoverPresentationController?.permittedArrowDirections = UIPopoverArrowDirection(rawValue: 0)
            popoverPresentationController?.sourceView = vMainTabBar?.view
            var rect: CGRect = vMainTabBar!.view.frame
            
            rect.origin.x = vMainTabBar!.view.frame.size.width / 20
            rect.origin.y = vMainTabBar!.view.frame.size.height / 20
            
            mIncomingCallAlert?.popoverPresentationController?.sourceRect = rect
            mIncomingCallAlert?.popoverPresentationController?.sourceView = vMainTabBar?.view
        }
        vMainTabBar?.present(mIncomingCallAlert!, animated: true, completion: nil)
        self.NotifyIncomingCall(sTitle: sTitle)
    }
    
    func HideAlert() ->Void
    {
        if mIncomingCallAlert == nil
        {
            return
        }
        
        mIncomingCallAlert?.dismiss(animated: true, completion: nil)
        mIncomingCallAlert = nil
    }
    
    private func NotifyIncomingCall(sTitle: String) ->Void
    {
        let Notify: UILocalNotification = UILocalNotification()
        
        Notify.alertBody = NSString(format: "Incoming Call: %@", sTitle) as String
        
        Notify.alertAction = "Answer"
        Notify.soundName = "oldphone-mono-30s.caf"
        
        UIApplication.shared.presentLocalNotificationNow(Notify)
    }
    

}
