
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

class cSettingVideoCodecTableView: UITableViewController
{
    
    let SECTION_TOTAL_COUNT     = 1
    let ROW_TOTAL_COUNT         = cVaxUserAgentInfo.VAX_VIDEO_CODEC_TOTAL
    
    let SECTION_ID_VIDEO_CODEC  = 0
    
    var mCodecNo: [AnyHashable] = []
    var mCodecEnabled: [Bool] = []

    override func viewDidLoad()
    {
        super.viewDidLoad()
        
        mCodecEnabled = [Bool](repeating: true, count :cVaxUserAgentInfo.VAX_VIDEO_CODEC_TOTAL)
        
        self.tableView.setEditing(true, animated: true)
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    static func PostInitialize() ->Void
    {
        ApplyVideoCodec()
    }
    
    static func ResetData() ->Void
    {
        let vStore: cStoreVideoCodecs = cStoreVideoCodecs()
        vStore.ResetData()
        
        PostInitialize()
    }
    
    static func ApplyVideoCodec() ->Void
    {
        var aCodecNo: [AnyHashable] = [AnyHashable]()
        
        let vStore: cStoreVideoCodecs = cStoreVideoCodecs()
        vStore.GetVideoCodec(&aCodecNo)
        
        cVaxPhoneCodec.GetSelf().ActivateVideoCodec(false, -1)
        
        for nCount in 0..<cVaxUserAgentInfo.VAX_VIDEO_CODEC_TOTAL
        {
            let nCodecNo: Int = aCodecNo[nCount] as! Int
            let bEnabled: Bool = vStore.GetVideoCodecEnabled(nCodecNo)
            
            cVaxPhoneCodec.GetSelf().ActivateVideoCodec(bEnabled, nCodecNo)
        }
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
        
    override func viewWillAppear(_ animated: Bool)
    {
        mCodecNo = [AnyHashable]()
        
        let vStore: cStoreVideoCodecs = cStoreVideoCodecs()
        vStore.GetVideoCodec(&mCodecNo)
        
        cVaxPhoneCodec.GetSelf().ActivateVideoCodec(false, -1)
        
        for nCount in 0..<cVaxUserAgentInfo.VAX_VIDEO_CODEC_TOTAL
        {
            let nCodecNo: Int = mCodecNo[nCount] as! Int
            mCodecEnabled[nCodecNo] = vStore.GetVideoCodecEnabled(nCodecNo)
            
            cVaxPhoneCodec.GetSelf().ActivateVideoCodec(mCodecEnabled[nCodecNo], nCodecNo)
        }
        
        self.tableView.reloadData()
        super.viewWillAppear(animated)
    }

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
        
    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int
    {
        return SECTION_TOTAL_COUNT
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int
    {
        return ROW_TOTAL_COUNT
    }
    
    func GenCellName(nCodecNo: Int) ->String
    {
        return "VideoCodecVP8CellId"
    }
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell 
     {
        let nCodecNo: Int = mCodecNo[indexPath.row] as! Int
        let sCellName: String = self.GenCellName(nCodecNo: nCodecNo)
        
        let vRowView: cSettingVideoCodecRowView  = tableView.dequeueReusableCell(withIdentifier: sCellName, for: indexPath) as! cSettingVideoCodecRowView

        let bCodecEnabled: Bool = mCodecEnabled[nCodecNo]
        vRowView.SetCodecEnabled(bEnable: bCodecEnabled)
        
        return vRowView
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    override func tableView(_ tableView: UITableView, canMoveRowAt indexPath: IndexPath) -> Bool
    {
        return false
    }
    
    override func tableView(_ tableView: UITableView, editingStyleForRowAt indexPath: IndexPath) -> UITableViewCell.EditingStyle
    {
        return .none
    }
    
    override func tableView(_ tableView: UITableView, shouldIndentWhileEditingRowAt indexPath: IndexPath) -> Bool
    {
        return false
    }
    //////////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////////
    
    override func tableView(_ tableView: UITableView, moveRowAt sourceIndexPath: IndexPath, to destinationIndexPath: IndexPath)
    {
        let nSrcRowNo: Int = sourceIndexPath.row
        let nDscRowNo: Int = destinationIndexPath.row
        
        let vCodecNo: NSNumber = mCodecNo[nSrcRowNo] as! NSNumber
        mCodecNo.remove(at: nSrcRowNo)
        mCodecNo.insert(vCodecNo, at: nDscRowNo)
        
        let vStore: cStoreVideoCodecs = cStoreVideoCodecs()
        vStore.SetVideoCodec(&mCodecNo)
        
        cSettingVideoCodecTableView.ApplyVideoCodec()
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

    @IBAction func OnClickVideoCodecVP8(_ sender: Any)
    {
        let vCheckBox: cCustomCellCheckBox = sender as! cCustomCellCheckBox
        let bEnabled: Bool = vCheckBox.GetState()
        
        mCodecEnabled[cVaxUserAgentInfo.VAX_VIDEO_CODEC_VP8] = bEnabled
        
        let vStore: cStoreVideoCodecs = cStoreVideoCodecs()
        vStore.SetVideoCodecEnabled(cVaxUserAgentInfo.VAX_VIDEO_CODEC_VP8, bEnabled)
        
        cVaxPhoneCodec.GetSelf().ActivateVideoCodec(bEnabled, cVaxUserAgentInfo.VAX_VIDEO_CODEC_VP8)
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
}
