 
 //  Created by VaxSoft
 //  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

class cSettingVoiceCodecTableView: UITableViewController
{
    let  SECTION_TOTAL_COUNT       = 1
    let  ROW_TOTAL_COUNT           = cVaxUserAgentInfo.VAX_AUDIO_CODEC_TOTAL
    
    var mCodecNo: [AnyHashable] = []
    var mCodecEnabled: [Bool] = []

    override func viewDidLoad()
    {
        super.viewDidLoad()
        
        mCodecEnabled = [Bool](repeating: true, count:5)

        self.tableView.setEditing(true, animated: true)
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

    static func PostInitialize() ->Void
    {
        cSettingVoiceCodecTableView.ApplyVoiceCodec()
    }
    
    static func ResetData() ->Void
    {
        let vStore: cStoreVoiceCodecs = cStoreVoiceCodecs()
        vStore.ResetData()
        
        cSettingVoiceCodecTableView.PostInitialize()
    }
    
    static func ApplyVoiceCodec() ->Void
    {
        var aCodecNo: [AnyHashable] = [AnyHashable]()
        
        let vStore: cStoreVoiceCodecs = cStoreVoiceCodecs()
        vStore.GetVoiceCodec(&aCodecNo)
        
        cVaxPhoneCodec.GetSelf().ActivateAudioCodec(false, -1)
        
        for nCount in 0..<cVaxUserAgentInfo.VAX_AUDIO_CODEC_TOTAL
        {
            let nCodecNo: Int = aCodecNo[nCount] as! Int
            let bEnabled: Bool = vStore.GetVoiceCodecEnabled(nCodecNo)
            
            cVaxPhoneCodec.GetSelf().ActivateAudioCodec(bEnabled, nCodecNo)
        }
    }

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    override func viewWillAppear(_ animated: Bool)
    {
        mCodecNo = [AnyHashable]()
        
        let vStore: cStoreVoiceCodecs = cStoreVoiceCodecs()
        vStore.GetVoiceCodec(&mCodecNo)
        
        cVaxPhoneCodec.GetSelf().ActivateVideoCodec(false, -1)
        
        for nCount in 0..<cVaxUserAgentInfo.VAX_AUDIO_CODEC_TOTAL
        {
            let nCodecNo: Int = mCodecNo[nCount] as! Int
            mCodecEnabled[nCodecNo] = vStore.GetVoiceCodecEnabled(nCodecNo)
            
            cVaxPhoneCodec.GetSelf().ActivateAudioCodec(mCodecEnabled[nCodecNo], nCodecNo)
        }
        
        self.tableView.reloadData()
        super.viewWillAppear(animated)
        
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int
    {
        return SECTION_TOTAL_COUNT
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int
    {
        return ROW_TOTAL_COUNT
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    func GenCellName(_ nCodecNo: Int) ->String
    {
        if nCodecNo == cVaxUserAgentInfo.VAX_AUDIO_CODEC_G711U
        {
            return "VoiceCodecG711uCellId"
        }
        
        if nCodecNo == cVaxUserAgentInfo.VAX_AUDIO_CODEC_G711A
        {
            return "VoiceCodecG711aCellId"
        }
        
        if nCodecNo == cVaxUserAgentInfo.VAX_AUDIO_CODEC_G729
        {
            return "VoiceCodecG729CellId"
        }
        
        if nCodecNo == cVaxUserAgentInfo.VAX_AUDIO_CODEC_GSM610
        {
            return "VoiceCodecGSMCellId"
        }
        
        return "VoiceCodeciLBCCellId";
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell
     {
        let nCodecNo: Int = mCodecNo[indexPath.row] as! Int
        let sCellName: String = self.GenCellName(nCodecNo)
        
        let vRowView: cSettingVoiceCodecRowView  = tableView.dequeueReusableCell(withIdentifier: sCellName, for: indexPath) as! cSettingVoiceCodecRowView

        let bCodecEnabled: Bool = mCodecEnabled[nCodecNo]
        vRowView.SetCodecEnabled(bEnable: bCodecEnabled)
        
        return vRowView
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    override func tableView(_ tableView: UITableView, canMoveRowAt indexPath: IndexPath) -> Bool
    {
        return true
    }
    
    override func tableView(_ tableView: UITableView, editingStyleForRowAt indexPath: IndexPath) -> UITableViewCell.EditingStyle
    {
        return .none
    }
    
    override func tableView(_ tableView: UITableView, shouldIndentWhileEditingRowAt indexPath: IndexPath) -> Bool
    {
        return false
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    override func tableView(_ tableView: UITableView, moveRowAt sourceIndexPath: IndexPath, to destinationIndexPath: IndexPath)
    {
        let nSrcRowNo: Int = sourceIndexPath.row
        let nDscRowNo: Int = destinationIndexPath.row
        
        let vCodecNo: NSNumber = mCodecNo[nSrcRowNo] as! NSNumber
        mCodecNo.remove(at: nSrcRowNo)
        mCodecNo.insert(vCodecNo, at: nDscRowNo)
        
        let vStore: cStoreVoiceCodecs = cStoreVoiceCodecs()
        vStore.SetVoiceCodec(mCodecNo)
        
        cSettingVoiceCodecTableView.ApplyVoiceCodec()
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

    @IBAction func OnClickVideoCodecG711u(_ sender: Any)
    {
        let vCheckBox: cCustomCellCheckBox = sender as! cCustomCellCheckBox
        let bEnabled: Bool = vCheckBox.GetState()
        
        mCodecEnabled[cVaxUserAgentInfo.VAX_AUDIO_CODEC_G711U] = bEnabled
        
        let vStore: cStoreVoiceCodecs = cStoreVoiceCodecs()
        vStore.SetVoiceCodecEnabled(cVaxUserAgentInfo.VAX_AUDIO_CODEC_G711U, bEnabled)
        
        cVaxPhoneCodec.GetSelf().ActivateAudioCodec(bEnabled, cVaxUserAgentInfo.VAX_AUDIO_CODEC_G711U)
    }
    
    @IBAction func OnClickVideoCodecG711a(_ sender: Any)
    {
        let vCheckBox: cCustomCellCheckBox = sender as! cCustomCellCheckBox
        let bEnabled: Bool = vCheckBox.GetState()
        
        mCodecEnabled[cVaxUserAgentInfo.VAX_AUDIO_CODEC_G711A] = bEnabled
        
        let vStore: cStoreVoiceCodecs = cStoreVoiceCodecs()
        vStore.SetVoiceCodecEnabled(cVaxUserAgentInfo.VAX_AUDIO_CODEC_G711A, bEnabled)
        
        cVaxPhoneCodec.GetSelf().ActivateAudioCodec(bEnabled, cVaxUserAgentInfo.VAX_AUDIO_CODEC_G711A)
    }
    
    @IBAction func OnClickVideoCodecG729(_ sender: Any)
    {
        let vCheckBox: cCustomCellCheckBox = sender as! cCustomCellCheckBox
        let bEnabled: Bool = vCheckBox.GetState()
        
        mCodecEnabled[cVaxUserAgentInfo.VAX_AUDIO_CODEC_G729] = bEnabled
        
        let vStore: cStoreVoiceCodecs = cStoreVoiceCodecs()
        vStore.SetVoiceCodecEnabled(cVaxUserAgentInfo.VAX_AUDIO_CODEC_G729, bEnabled)
                        
        cVaxPhoneCodec.GetSelf().ActivateAudioCodec(bEnabled, cVaxUserAgentInfo.VAX_AUDIO_CODEC_G729)
    }
    
    @IBAction func OnClickVideoCodecGSM(_ sender: Any)
    {
        let vCheckBox: cCustomCellCheckBox = sender as! cCustomCellCheckBox
        let bEnabled: Bool = vCheckBox.GetState()
        
        mCodecEnabled[cVaxUserAgentInfo.VAX_AUDIO_CODEC_GSM610] = bEnabled
        
        let vStore: cStoreVoiceCodecs = cStoreVoiceCodecs()
        vStore.SetVoiceCodecEnabled(cVaxUserAgentInfo.VAX_AUDIO_CODEC_GSM610, bEnabled)
        
        cVaxPhoneCodec.GetSelf().ActivateAudioCodec(bEnabled, cVaxUserAgentInfo.VAX_AUDIO_CODEC_GSM610)
    }
    
    @IBAction func OnClickVideoCodeciLBC(_ sender: Any)
    {
        let vCheckBox: cCustomCellCheckBox = sender as! cCustomCellCheckBox
        let bEnabled: Bool = vCheckBox.GetState()
        
        mCodecEnabled[cVaxUserAgentInfo.VAX_AUDIO_CODEC_ILBC] = bEnabled
        
        let vStore: cStoreVoiceCodecs = cStoreVoiceCodecs()
        vStore.SetVoiceCodecEnabled(cVaxUserAgentInfo.VAX_AUDIO_CODEC_ILBC, bEnabled)
              
        cVaxPhoneCodec.GetSelf().ActivateAudioCodec(bEnabled, cVaxUserAgentInfo.VAX_AUDIO_CODEC_ILBC)
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

}
