
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

var mpVideoRemoteView: cVideoRemoteView! = nil
var mbVideoRemoteHideDisplayWhenAppear: Bool = true
var mbVideoRemoteDisplayGravity: Int = 0
var mbVideoRemoteCallConnected:Bool = false

class cVideoRemoteView: UIViewController
{
    var mDisplayLayer: CALayer!
    
    var m_nLastWidth: Int = 0
    var m_nLastHeight: Int = 0
    
    var mBgColor: UIColor!
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        
        mDisplayLayer = CALayer()

        m_nLastWidth  = -1
        m_nLastHeight = -1
        
        mBgColor = self.view.backgroundColor!
    }
    
    override func viewWillAppear(_ animated: Bool)
    {
        mpVideoRemoteView = self
        mDisplayLayer.isHidden = mbVideoRemoteHideDisplayWhenAppear
        
        self.ApplyContentGravity()
        super.viewWillAppear(animated)
    }
    
    override func viewDidAppear(_ animated: Bool)
    {
        super.viewDidAppear(animated)
        self.view.layer.addSublayer(mDisplayLayer!)
    }
    
    override func viewWillDisappear(_ animated: Bool)
    {
        mpVideoRemoteView = nil
        super.viewWillDisappear(animated)
    }
    
    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    @objc func OnAdjustSize() ->Void
    {
        mDisplayLayer.bounds = CGRect(x: 0, y: 0, width: view.frame.size.width, height: view.frame.size.height)
        mDisplayLayer.position = CGPoint(x: view.frame.size.width / 2.0, y: view.frame.size.height / 2.0)
        self.ApplyContentGravity()
    }
    
    func AdjustSize(bAfterDelay: Bool)
    {
        NSObject.cancelPreviousPerformRequests(withTarget: self, selector: #selector(self.OnAdjustSize), object: nil)
        
        if bAfterDelay
        {
            perform(#selector(self.OnAdjustSize), with: nil, afterDelay: 0.5)
        }
        else
        {
            self.OnAdjustSize()
        }
    }
    
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
    
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator)
    {
        self.AdjustSize(bAfterDelay: true)
    }
    
    func DisplayFrame(_ pDataRGB: UnsafeMutablePointer<Int8>!, _ nWidth: Int, _ nHeight: Int)
    {
        if m_nLastWidth != nWidth || m_nLastHeight != nHeight
        {
            self.AdjustSize(bAfterDelay: true)
            
            m_nLastWidth = nWidth
            m_nLastHeight = nHeight
        }
        
        let iColorSpace = CGColorSpaceCreateDeviceRGB()
        let iContext = CGContext(data: pDataRGB, width: nWidth, height: nHeight, bitsPerComponent: 8, bytesPerRow: nWidth * 4, space: iColorSpace, bitmapInfo: CGBitmapInfo.byteOrder32Little.rawValue | CGImageAlphaInfo.premultipliedFirst.rawValue)
        
        let iContextImage: CGImage? = iContext!.makeImage()
        mDisplayLayer.contents = iContextImage
    }
    
//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
    
    func OnConnectedCall() ->Void
    {
        self.view.backgroundColor = mBgColor
    }
    
    static func PostConnectedCall() ->Void
    {
        mbVideoRemoteCallConnected = true
        mbVideoRemoteHideDisplayWhenAppear = false
        
        if mpVideoRemoteView == nil
        {
            return
        }
        
        mpVideoRemoteView!.OnConnectedCall()
    }
    
//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
    
    func OnDisconnectedCall() ->Void
    {
        self.view.backgroundColor = mBgColor
        mDisplayLayer.isHidden = true
    }
    
    static func PostDisconnectedCall() ->Void
    {
        mbVideoRemoteCallConnected = false
        mbVideoRemoteHideDisplayWhenAppear = true
        
        if mpVideoRemoteView == nil
        {
            return
        }
        
        mpVideoRemoteView.OnDisconnectedCall()
    }
    
//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
    
    func OnPostedFrameRGB(_ pDataRGB: UnsafeMutablePointer<Int8>!, _ nSize: Int, _ nWidth: Int, _ nHeight: Int) ->Void
    {
        if mbVideoRemoteCallConnected
        {
            if mDisplayLayer!.isHidden
            {
                mDisplayLayer.isHidden = false
            }
        }
        
       mpVideoRemoteView!.DisplayFrame(pDataRGB, nWidth, nHeight)
    }
    
    static func PostFrameRGB(_ pDataRGB: UnsafeMutablePointer<Int8>!, _ nSize: Int, _ nWidth: Int, _ nHeight: Int)
    {
        if mpVideoRemoteView == nil
        {
            return
        }
        
        mpVideoRemoteView!.OnPostedFrameRGB(pDataRGB, nSize, nWidth, nHeight)
    }
    
//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
    
    let TOTAL_DISPLAY_GRAVITY_COUNT  = 4
        
    func ApplyContentGravity() ->Void
    {
        if mbVideoRemoteDisplayGravity < 0 || mbVideoRemoteDisplayGravity >= TOTAL_DISPLAY_GRAVITY_COUNT
        {
            mbVideoRemoteDisplayGravity = 0
        }
        
        let aGravity: [String] = [convertFromCALayerContentsGravity(CALayerContentsGravity.resizeAspect), convertFromCALayerContentsGravity(CALayerContentsGravity.resize), convertFromCALayerContentsGravity(CALayerContentsGravity.resizeAspectFill), convertFromCALayerContentsGravity(CALayerContentsGravity.center)]
        mDisplayLayer.contentsGravity = convertToCALayerContentsGravity(aGravity[mbVideoRemoteDisplayGravity])
        mDisplayLayer.masksToBounds = true
    }
    
    func AdjustContentGravity() ->Void
    {
        mbVideoRemoteDisplayGravity += 1
        self.ApplyContentGravity()
    }
    
    static func PostAdjustContentGravity() ->Void
    {
        if mpVideoRemoteView == nil
        {
            return
        }
        
        mpVideoRemoteView!.AdjustContentGravity()
    }
    
    @IBAction func OnDoubleTapView(_ sender: Any)
    {
        cVideoRemoteView.PostAdjustContentGravity()
    }


}

// Helper function inserted by Swift 4.2 migrator.
fileprivate func convertFromCALayerContentsGravity(_ input: CALayerContentsGravity) -> String {
	return input.rawValue
}

// Helper function inserted by Swift 4.2 migrator.
fileprivate func convertToCALayerContentsGravity(_ input: String) -> CALayerContentsGravity {
	return CALayerContentsGravity(rawValue: input)
}
