
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit

var mbVideoShowDialpadWhenAppear: Bool = false
var mvVideoView: cVideoView! = nil
var mvVideoViewStatusText: String! = nil

class cVideoView: UIViewController
{
    var mvTorch: cCameraTorch!
    @IBOutlet var mStatusText: UILabel!
    
    @IBOutlet var mVideoCheckBox: cCustomCellCheckBox!
    @IBOutlet var mSpeakerPhoneCheckBox: cCustomCellCheckBox!
    @IBOutlet var mMuteMicCheckBox: cCustomCellCheckBox!
    
    @IBOutlet var mCheckBoxCamTorch: cCustomCellCheckBox!
    
    @IBOutlet var mBtnVoiceChanger: UIButton!
    @IBOutlet var mBtnHangUp: UIButton!
    @IBOutlet var mBtnCamRotate: UIButton!

    override func viewDidLoad()
    {
        super.viewDidLoad()
         
        mvTorch = cCameraTorch()
        
        mVideoCheckBox.LoadCheckBox(nil, "avVideoOFF", "avVideoON")
        mMuteMicCheckBox.LoadCheckBox(nil, "avMicOFF", "avMicON")
        mSpeakerPhoneCheckBox.LoadCheckBox(nil, "avPhoneSpkOFF", "avPhoneSpkON")
        
        mCheckBoxCamTorch.LoadCheckBox(nil, "aFlashCamOFF", "aFlashCamON")
        self.HideBtnTorch(bHidden :true)
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
    }
    
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
    
    override func viewWillAppear(_ animated: Bool)
    {
        mvVideoView = self
        
        super.viewWillAppear(animated)
        
        if mbVideoShowDialpadWhenAppear
        {
            return
        }
        
        self.UpdateStates()
        self.HideBtnTorch(bHidden: true)
        
        self.ShowStatusMsg(sMsgText: mvVideoViewStatusText!)
        cVaxPhoneVideo.GetSelf().OpenVideoDevice()
    }
    
    override func viewDidAppear(_ animated: Bool)
    {
        super.viewDidAppear(animated)
        
        if mbVideoShowDialpadWhenAppear
        {
            mbVideoShowDialpadWhenAppear = false
            self.performSegue(withIdentifier: "SegueDialPadView", sender: self)
            
            return
        }
        
        self.ShowWithAutoHide()
    }
    
    override func viewWillDisappear(_ animated: Bool)
    {
        mvVideoView = nil
        
        if !cVaxPhoneGeneral.GetSelf().IsLineConnected()
        {
            cVaxPhoneVideo.GetSelf().CloseVideoDevice()
        }
        
        super.viewWillDisappear(animated)
    }
    
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
    
    static func OnStartedAPP() ->Void
    {
        mvVideoViewStatusText = ""
        cDialPadView.OnStartedAPP()
    }
    
    static func ShowDialpadWhenAppear(sNoText: String) ->Void
    {
        if  NSString(string: sNoText).length == 0
        {
            mbVideoShowDialpadWhenAppear = false
            return
        }
        
        cDialPadView.PostPhoneNoText(sNoText: sNoText)
        mbVideoShowDialpadWhenAppear = true
    }
    
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
    
    static func PostInitialize() ->Void
    {
        if mvVideoView != nil
        {
            cVaxPhoneVideo.GetSelf().OpenVideoDevice();
        }
    }
    
    static func PostDialCallStarted() ->Void
    {
        cDialPadView.PostDialCallStarted()
    }
    
    static func PostDialCallFailed(nStatusCode: Int, sReasonPhrase: String)
    {
        cDialPadView.PostDialCallFailed(nStatusCode: nStatusCode, sReasonPhrase: sReasonPhrase)
    }
    
    static func ResetData() ->Void
    {
        mvVideoViewStatusText = ""
    }
    
    static func PostStatusText(_ sMsgText: String) ->Void
    {
        cDialPadView.PostStatusText(sMsgText: sMsgText)
        
        mvVideoViewStatusText = sMsgText
        
        if mvVideoView != nil
        {
            mvVideoView.ShowStatusMsg(sMsgText: sMsgText)
        }
    }
    
    static func PostConnectedCall() ->Void
    {
        cVaxPhoneVideo.GetSelf().OpenVideoDevice()
        
        cDialPadView.PostConnectedCall()
        cVideoRemoteView.PostConnectedCall()
    }
    
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
    
    static func PostDisconnectedCall() ->Void
    {
        let vTorch: cCameraTorch = cCameraTorch()
        vTorch.SwitchTorchMode(false)
        
        cDialPadView.PostDisconnectedCall()
        cVideoRemoteView.PostDisconnectedCall()
    }
    
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
    
    static func PostRemoteFrameRGB(_ pFrameRGB: UnsafeMutablePointer<Int8>!, _ nFrameSize: Int, _ nFrameWidth: Int, _ nFrameHeight: Int) ->Void
    {
        cVideoRemoteView.PostFrameRGB(pFrameRGB, nFrameSize, nFrameWidth, nFrameHeight)
    }
    
    static func PostDeviceFrameRGB(_ pFrameRGB: UnsafeMutablePointer<Int8>!, _ nFrameSize: Int, _ nFrameWidth: Int, _ nFrameHeight: Int) ->Void
    {
        cVideoDeviceView.PostFrameRGB(pFrameRGB, nFrameSize, nFrameWidth, nFrameHeight)
    }
    
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
    
    func ShowStatusMsg(sMsgText: String) ->Void
    {
        mStatusText.text = sMsgText
    }
    
    func UpdateStates() ->Void
    {
        let bMute: Bool = cVaxPhoneMicSpk.GetSelf().IsMuteMic()
        mMuteMicCheckBox.SetState(bMute)
        
        var bEnable: Bool = cVaxPhoneGeneral.GetSelf().IsSpeakerPhone()
        mSpeakerPhoneCheckBox.SetState(bEnable)
        
        bEnable = cVaxPhoneVideo.GetSelf().IsVideoEnabled()
        mVideoCheckBox.SetState(bEnable)
    }
    
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?)
    {
        self.ShowWithAutoHide()
        cVideoDeviceView.ShowView()
    }
    
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
    
    @IBAction func OnClickVideo(_ sender: Any)
    {
        let bEnable: Bool = mVideoCheckBox.GetState()
        cVaxPhoneVideo.GetSelf().EnableVideo(bEnable)
        
        self.ShowWithAutoHide()
    }

    @IBAction func OnClickVoiceChanger(_ sender: Any)
    {
        var bEnable: Bool = false
        
        if cVaxPhoneVoiceChanger.GetSelf().IsVoiceChangerEnabled()
        {
            bEnable = false
        }
        else
        {
            bEnable = true
        }
        
        cVaxPhoneVoiceChanger.GetSelf().VoiceChangerEnabled(bEnable)
        self.ShowWithAutoHide()
    }
    
    @IBAction func OnClickHangup(_ sender: Any)
    {
        cVaxPhoneSIP.GetSelf().DisconnectCall()
    }
    
    @IBAction func OnClickMuteMic(_ sender: Any)
    {
        let bMute: Bool = mMuteMicCheckBox.GetState()
        cVaxPhoneMicSpk.GetSelf().MuteMic(bMute)
        
        self.ShowWithAutoHide()
    }
    
    @IBAction func OnClickSpeakerPhone(_ sender: Any)
    {
        let bEnable: Bool = mSpeakerPhoneCheckBox.GetState()
        cVaxPhoneGeneral.GetSelf().SpeakerPhone(bEnable)
        
        self.ShowWithAutoHide()
    }
    
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
    
    @objc func OnShowWithAutoHide() ->Void
    {
        mVideoCheckBox.isHidden = true
        mMuteMicCheckBox.isHidden = true
        mSpeakerPhoneCheckBox.isHidden = true
        
        mBtnCamRotate.isHidden = true
        self.HideBtnTorch(bHidden: true)
        
        mBtnVoiceChanger.isHidden = true
        mBtnHangUp.isHidden = true
    }
    
    func ShowWithAutoHide() ->Void
    {
        mVideoCheckBox.isHidden = false
        mMuteMicCheckBox.isHidden = false
        mSpeakerPhoneCheckBox.isHidden = false
        
        mBtnCamRotate.isHidden = false
        self.HideBtnTorch(bHidden: false)
        
        mBtnVoiceChanger.isHidden = false
        mBtnHangUp.isHidden = false
        
        self.UpdateStates()
        
        NSObject.cancelPreviousPerformRequests(withTarget: self, selector: #selector(self.OnShowWithAutoHide), object: nil)
        perform(#selector(self.OnShowWithAutoHide), with: nil, afterDelay: 4.0)
    }
    
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
    
    @IBAction func OnClickLogout(_ sender: Any)
    {
        cVaxPhoneSIP.GetSelf().DisconnectCall()
        cMainTabBarController.DismissMainTabBar()
    }
    
    @IBAction func OnClickRotateCamera(_ sender: Any)
    {
        cVaxPhoneVideo.GetSelf().RotateVideoDevice()
        self.ShowWithAutoHide()
    }
    
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
    
    @IBAction func OnClickCameraTorch(_ sender: Any)
    {
        mvTorch.SwitchTorchMode(mCheckBoxCamTorch.GetState())
        self.ShowWithAutoHide()
    }
    
    func HideBtnTorch(bHidden: Bool) ->Void
    {
        if !mvTorch.IsTorchAvailable()
        {
            mCheckBoxCamTorch.isHidden = true
            return
        }
        
        mCheckBoxCamTorch.isHidden = bHidden
        mCheckBoxCamTorch.SetState(mvTorch.IsTorchModeON())
    }
    
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
    
    @IBAction func UnwindFromDialpadView(_ segue: UIStoryboardSegue)
    {
        
    }
}
