
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import UIKit


let DATABASE_NAME = "VaxPhone.db"

let SQLITE_STATIC = unsafeBitCast(0, to: sqlite3_destructor_type.self)
let SQLITE_TRANSIENT = unsafeBitCast(-1, to: sqlite3_destructor_type.self)

class cDataSQL: NSObject
{
    var m_pDatabase: OpaquePointer? = nil
    var m_pCompiledStatement: OpaquePointer? = nil
    
    static func CheckAndCreateDatabase()
    {
        let fileManager = FileManager.default
        
        let documentsUrl = fileManager.urls(for: .documentDirectory, in: .userDomainMask)
        
        guard documentsUrl.count != 0
            
        else
        {
            return
        }
        
        let finalDatabaseURL = documentsUrl.first!.appendingPathComponent(DATABASE_NAME)
        
        if !( (try? finalDatabaseURL.checkResourceIsReachable()) ?? false)
        {
            print("DB does not exist in documents folder")
            
            let documentsURL = Bundle.main.resourceURL?.appendingPathComponent(DATABASE_NAME)
            
            do
            {
                try fileManager.copyItem(atPath: (documentsURL?.path)!, toPath: finalDatabaseURL.path)
            }
                
            catch let error as NSError
            {
                print("Couldn't copy file to final location! Error:\(error.description)")
            }
            
        } else
        {
            print("Database file found at path: \(finalDatabaseURL.path)")
        }
    }
    
    override init()
    {
        super.init()
    }
    
    deinit
    {

    }
    
    func OpenDatabase() -> Bool
    {
        let vPathDatabaseSQL = try! FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: false).appendingPathComponent(DATABASE_NAME)
        
        if sqlite3_open(vPathDatabaseSQL.path, &m_pDatabase) != SQLITE_OK
        {
            print("error opening database")
            return false;
        }
        
        return true;
    }

    
    func CloseDatabase()
    {
        if m_pDatabase == nil
        {
            return
        }
        
        sqlite3_close(m_pDatabase)
        m_pDatabase = nil
    }
    
    private func GetColumnData(_ pCompiledStament: OpaquePointer, _ nColNo: Int) -> Any!
    {
        let nType: Int = Int(sqlite3_column_type(pCompiledStament, Int32(nColNo)))
        
        if nType == Int(SQLITE_TEXT)
        {
            let sText: String = String(cString: sqlite3_column_text(pCompiledStament, Int32(nColNo)))
            return sText
        }
        
        if nType == Int(SQLITE_INTEGER)
        {
            return id_t(Int(sqlite3_column_int(pCompiledStament, Int32(nColNo))))
        }
        
        if nType == Int(SQLITE_BLOB)
        {
            let vBlobData = sqlite3_column_blob(pCompiledStament, Int32(nColNo))
            let nBlobLength: Int = Int(sqlite3_column_bytes(pCompiledStament, Int32(nColNo)))
            
            return NSData(bytes: vBlobData, length: Int(nBlobLength))
        }
        
        return nil
    }
    
    func StartQuery(_ sQuery: String, _ aParam: [Any]!) -> Bool
    {
        if sqlite3_prepare_v2(m_pDatabase, (sQuery as NSString).utf8String, -1, &m_pCompiledStatement, nil) != SQLITE_OK
        {
            return false
        }
        
        for nParamCount: Int in 0..<aParam.count
        {
            let iParam: Any = aParam[nParamCount]
            
            if (iParam is String)
            {
                let sParamText: String = (iParam as! String)
                sqlite3_bind_text(m_pCompiledStatement, Int32(nParamCount + 1), (sParamText as NSString).utf8String, -1, nil)
            }
            else if (iParam is NSNumber)
            {
                let vParamNumber = (iParam as! NSNumber)
                sqlite3_bind_int(m_pCompiledStatement, Int32(nParamCount + 1), CInt(truncating: vParamNumber))
            }
                
            else if (iParam is NSData)
            {
                let vParamData: NSData = (iParam as! NSData)
                sqlite3_bind_blob(m_pCompiledStatement, Int32(nParamCount + 1), vParamData.bytes, Int32(Int(vParamData.length)), SQLITE_TRANSIENT)
            }
        }
        
        return true
    }

    
    func StartQuery(_ sQuery: String) -> Bool
    {
        let Result = sqlite3_prepare_v2(m_pDatabase, (sQuery as NSString).utf8String, -1, &m_pCompiledStatement, nil)
        if Result != SQLITE_OK
        {
            return false
        }
        
        return true
    }
    
    func EndQuery()
    {
        if m_pCompiledStatement == nil
        {
            return
        }
        
        sqlite3_finalize(m_pCompiledStatement)
        m_pCompiledStatement = nil
    }
    
    func GetRowData(_ aRowDB: inout [Any]) -> Bool
    {
        if m_pCompiledStatement == nil
        {
            return false
        }
        
        let nCount = sqlite3_step(m_pCompiledStatement)
        
        if nCount != SQLITE_ROW
        {
            return false
        }
        
        var nColumnCount: Int = 0
        
        while true
        {
            var ColumnData: Any!
            
            ColumnData = self.GetColumnData(m_pCompiledStatement!, nColumnCount)
            if ColumnData == nil
            {
                break
            }
            
            aRowDB.append(ColumnData)
            nColumnCount += 1
        }
        return true
    }
    
    private func ShowRecord(_ pQuery: String) -> Bool
    {
        return true
    }
    
}
