
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

import PushKit

@objc protocol iVaxPushKit: NSObjectProtocol
{
    @objc optional func OnVaxTryingPushKit() ->Void
    @objc optional func OnVaxSuccessPushKit(_ sDeviceToken: String) ->Void
    @objc optional func OnVaxFailedPushKit() ->Void
    @objc optional func OnVaxNotificationPushKit(_ sCallerName: String, _ sCallerId: String) ->Void
}

class cVaxPushKit: NSObject, PKPushRegistryDelegate
{
    var mPushRegistry: PKPushRegistry!
    var mPushKitDelegate: iVaxPushKit!
        
    init(iDelegate: Any)
    {
        mPushKitDelegate = (iDelegate as! iVaxPushKit)
        
        super.init()
    }
    
    deinit
    {

    }
    
    func Start()
    {
        mPushRegistry = PKPushRegistry(queue: DispatchQueue.main)
        
        mPushRegistry.delegate = self
        mPushRegistry.desiredPushTypes = [PKPushType.voIP]
        
        if !mPushKitDelegate.responds(to: #selector(iVaxPushKit.OnVaxTryingPushKit))
        {
            return;
        }
        
        perform(#selector(OnVaxTryingPushKit), with: self, afterDelay: 0.0);
    }
    
    /////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    @objc func OnVaxTryingPushKit() ->Void
    {
        mPushKitDelegate.OnVaxTryingPushKit!()
    }
    
    @objc func OnVaxSuccessPushKit(_ sToken: String) ->Void
    {
        mPushKitDelegate.OnVaxSuccessPushKit!(sToken)
    }

    /////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////
    
    func pushRegistry(_ registry: PKPushRegistry, didUpdate credentials: PKPushCredentials, for type: PKPushType)
    {
        if !mPushKitDelegate.responds(to: #selector(iVaxPushKit.OnVaxSuccessPushKit))
        {
            return;
        }
        
        let sToken = credentials.token.map { String(format: "%02x", $0) }.joined()
                        
        perform(#selector(OnVaxSuccessPushKit), with: sToken, afterDelay: 1.0);
    }
        
    func pushRegistry(_ registry: PKPushRegistry, didInvalidatePushTokenFor type: PKPushType)
    {
        if !mPushKitDelegate.responds(to: #selector(iVaxPushKit.OnVaxFailedPushKit))
        {
            return;
        }
        
        mPushKitDelegate.OnVaxFailedPushKit!()
    }
    
    ////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////
    
    func pushRegistry(_ registry: PKPushRegistry, didReceiveIncomingPushWith payload: PKPushPayload, for type: PKPushType, completion: @escaping () -> Void)
    {
        if !mPushKitDelegate.responds(to: #selector(iVaxPushKit.OnVaxNotificationPushKit))
        {
            return;
        }
        
        let vCallInfo = payload.dictionaryPayload["aps"] as? Dictionary<String, String>
        
        let vCallerName = vCallInfo?["CallerName"]
        let vCallerId = vCallInfo?["CallerId"]
        
        var sCallerName: String = "";
        var sCallerId: String = "";
        
        if vCallerName != nil
        {
            sCallerName = vCallerName!;
        }
        
        if vCallerId != nil
        {
            sCallerId = vCallerId!;
        }
        
        mPushKitDelegate.OnVaxNotificationPushKit!(sCallerName, sCallerId)
    }
  
    ////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////
}
